;+
; Name: fermi_get_gti
; 
; Purpose:  Get times of valid Fermi data (not in SAA).  This is available in the GTI extension of the
;   daily cspec pha files.
; 
; Calling sequence: times = fermi_get_gti(date)
; 
; Input:
;   date - either a single time (assumes entire day), or a time range, in anytim
;          format (if seconds then sec since 1979/1/1).  Will do whole days.
;          
; Input Keywords:
;   cleanup - if set, remove any files copied.  Default is 1.
;   
; Output Keywords:
;   status - 0 / 1 means fail /success
;   
; Examples:
;   ptim, fermi_get_gti('12-dec-2015', cleanup=0)
;   ptim, fermi_get_gti(['12-dec-2015', '15-dec-2015 23:00'], status=status)
;   
; Written: Kim Tolbert, 25-Feb-2016
;
;-

function fermi_get_gti, date, cleanup=cleanup, status=status

checkvar, cleanup, 1

status = 0

time_range = n_elements(date) eq 1 ? anytim(date, /date_only) + [0,86400.d0] : anytim(date)
ndays = ceil((time_range[1] - anytim(time_range[0], /date_only)) / 86400.d0)

; Sometimes a file for a detector is missing.  Since we can use file for any detector, find the first
; det that has all the files for times requested. Or if none, find the det with the most files.
dets = ['b0','b1','n0','n1','n2','n3','n4','n5','n6','n7','n8','n9','na','nb']
count_det = intarr(14)
for idet =0,13 do begin
  gbm_find_data, date=time_range, pattern='_cspec_'+dets[idet], count=count, file=file, copy=0
  if count eq ndays then goto, getfiles
  count_det[idet] = count
endfor
if max(count_det,idet) eq 0 then begin
  message, /cont, 'No CSPEC PHA files found for times requested. Aborting.'
  return, -1
endif

getfiles:
gbm_find_data, date=time_range, pattern='_cspec_'+dets[idet], count=count, file=file
for ifile=0,count-1 do begin
  gti = mrdfits(file[ifile], 3, ghdr, /silent, /dscale)
  ref_utc = mjd2any(fxpar(ghdr, 'MJDREFI'))
  g1 = ref_utc + gti.start
  g2 = ref_utc + gti.stop
  
  if ifile eq 0 then begin
    gstart = g1
    gend = g2
  endif else begin
    ; After first file, merge last interval already saved with first new one, if they overlap
    if g1[0] lt last_item(gend) then begin
      gend[n_elements(gend)-1] = g2[0]
      g1 = g1[1:*]
      g2 = g2[1:*]
    endif
    gstart = [gstart, g1]
    gend   = [gend, g2]
  endelse
  
endfor
tgood = transpose([[gstart],[gend]])

if cleanup then file_delete, file, /quiet

status = 1
return, tgood
end
