;+
;
; NAME:
;   gbm_fix_counter_overflow, spec
;
; PURPOSE:
;  Corrects 2 byte counter overflow in the gbm spectral files
;
;
; CATEGORY:
;       SPECTRAL FITTING, SPEX - messenger
;
; CALLING SEQUENCE:
;
; CALLS:
;
; INPUTS:
;	Spec - gbm count rate structure from pha file, only overflow intervals are changed
;	Spec is obtained from mrdfits( phafile, 2, /dscale)
;	dscale must be used or counts will be a 2-byte integer field that can't
;	hold the corrected value properly
;	** Structure <7c92700>, 5 tags, length=1048, data length=1046, refs=1:
;	   COUNTS          DOUBLE    Array[128]
;	   EXPOSURE        FLOAT           4.08290
;	   QUALITY         INT              0
;	   TIME            DOUBLE      3.1930713e+008
;	   ENDTIME         DOUBLE      3.1930713e+008
;	Ebounds - structure with energy calibration
;		IDL> ebounds = mrdfits(f,1,/fscale)
;		MRDFITS: Binary table.  3 columns by  128 rows.
;		IDL> help, ebounds,/st
;		** Structure <d3227c0>, 3 tags, length=12, data length=10, refs=1:
;		   CHANNEL         INT              0
;		   E_MIN           FLOAT           4.60264
;		   E_MAX           FLOAT           5.52407

;
; OUTPUTS:
;

; PROCEDURE:
;	Overflows appear in the spec.counts field and the exposure is also incorrect
;	when there is overflow.  This proc uses jumper to analyze and correct these oveflows
;	and then uses the corrected counts to compute exposure assuming a deadtime of 2.6 e-6 seconds per event
;
; Modification History:
;
; 18-feb-2011 richard.schwartz@nasa.gov
; 30-jul-2012, richard.schwartz@nasa.gov, add indexing with p to dt
;------------------------------------------------------------------------------
pro gbm_fix_counter_overflow, spec, ebounds

if size(spec.counts,/tname) eq 'INT' or size(spec.time,/tname) ne 'DOUBLE' then begin
	message,/info, ' SPEC extension must be obtained using mrdfits(phafile,2,/dscale). Returning'
	return
	endif
dt = spec.endtime - spec.time
n=n_elements(spec)

oldcounts   = spec.counts
counts = gbm_correct_overflow( spec, ebounds)

p = where( total(oldcounts,1) ne total(counts,1), np)

spec.counts = counts
;if np gt 0 then spec[p].exposure = dt[-2.6e-6*total(counts[*,p],1)
if np gt 0 then spec[p].exposure = dt[p]-2.6e-6*total(counts[*,p],1)

end
