;+
; Name: gbm_get_ephem
;
; Purpose: Return the ephemeris data for Fermi GBM detectors for requested times
;
; Method: Locates the poshist file(s) for requested time either already in your current directory, or 
;   copies it (them) from the fermi gbm archive.  Calls gbm_rd_poshist to read poshist file(s).  If more
;   than one file, concatenates arrays, then extracts the structure elements that are between your 
;   requested times.
;   
; Arguments:
; date - Either a single time, or a range in anytim format (if sec, since 1979/1/1)
;
; Input Keywords:
;  None
; 
; Output Keywords:
; status - 0/1 means failure/success
; 
; Output:
; Returns an array of structures with ephemeris data.  The UTC field is seconds since 1-Jan-1979.
; 
; Example:
; 
;   pos = gbm_get_ephem(['11-jun-2010 12:00', '12-jun-2010 04:00'], status=status)
;
; Written: Kim Tolbert 27-Jul-2011
; Modifications:
;-
function gbm_get_ephem, date, status=status

status = 1

gbm_find_data, date=date, pattern='_poshist_',count=count, file=file
if count eq 0 then begin
  message,/cont,'Could not download poshist file from fermi gbm site. Aborting.'
  status = 0
  return, -1
endif

pos = gbm_rd_poshist(file[0], /utbase)
if count gt 1 then begin
  for ifile=1,count-1 do pos = [pos, gbm_rd_poshist(file[ifile], /utbase)]
endif

time = anytim(date)
ind = value_locate(pos.utc, time)
return, n_elements(ind) eq 1 ? pos[ind] : pos[ind[0]:ind[1]]

end

