;+
;Name: GBM_GET_LLT_VALUE
;
;Purpose: Find the lower-level disc threshold for the GBM NaI detectors for the
;	input date. Initially, only looking for values where the E threshold is 10 keV
;
;Input:
;	Date - find the LLT value at the start of this day (anytim readable date)
;
;	value = gbm_get_llt_value( date )
;		returns the LLT threshold in keV, initially 6 or 10 keV
;Keywords:
;	URL - default is 'http://fermi.gsfc.nasa.gov/FTP/glast/data/gbm/llt/glg_llt.csv'
;		    TABLE - parsed values of file at URL.  header is skipped
;			Columns - Table headers for each column
;			UT Date1	UT1	MET1	UT Date2	UT2	MET2	NaI Det. Range  (0-11)	LLT	Lowest Useable CSpec Channel (1-128)
;			Lowest Useable CTime Channel (1-8)	Approx. Energy Threshold (keV)	Purpose
;	UT_RANGE - actual time range found when threshold is not 6 keV the normal default,
;		if the threshold is 6 keV, this value is returned as -1
;History:
;	Written, richard.schwartz@nasa.gov
;	13-jul-2012, Version 1
;-
function gbm_get_llt_value, date, url=url, table =table, columns=columns, ut_range=ut_range

default, url,'http://fermi.gsfc.nasa.gov/FTP/glast/data/gbm/llt/glg_llt.csv'
default, ut_range, -1
sock_copy, url, dummy,err=err,out_dir=curdir(), local=local

out = 6.0 ;6 keV by default
table=rd_tfile(local,/auto,delim=',',hskip=1)
columns = str2arr(delim=',','UT Date1,	UT1,	MET1,	UT Date2,	UT2	MET2,	NaI Det. Range  (0-11),	LLT	Lowest Useable CSpec Channel (1-128),'+$
	'Lowest Useable CTime Channel (1-8),	Approx. Energy Threshold (keV),	Purpose')

;Energy threshold in column 11
ethresh = reform(table[10,*])
z10     = where( ethresh eq long(10), nz10)
if nz10 ge 1 then begin
	z10table = table[*,z10]
	null   = where( z10table eq '', nnull)
	if nnull ge 1 then z10table[3,null] = '2049-01-01'
	datlim = anytim( z10table[[0,3],*],/date)+86400.
	utdate = anytim(date)
	thisone = where( utdate ge datlim[0,*] and utdate lt datlim[1,*], nthis)
	if nthis eq 1  then begin
		out = float(ethresh[z10[thisone]])
		ut_range = anytim( (z10table[*,thisone])[[0,3]])
		endif
	endif
return, out
end

