;+
; PROJECT:
;	FERMI
; NAME:
;	GBM_QLOOK
;
; PURPOSE:
;	Builds the basic data structures with the GBM CSPEC rates and contains the
;	daynight and detector pointing to the Sun
;
; CATEGORY:
;	QUICKLOOK AND CATALOG GENERATION FOR FERMI GBM
;
; CALLING SEQUENCE:
;	gbm_qlook, daily_files,  data, synrate, $
;	 ERANGE=ERANGE, THRESHOLD=THRESHOLD, testscale=testscale, SMOOTHBIN=SMOOTHBIN

;
; CALLS:
;	none
;
; INPUTS:
;       Daily_files - directories containing CSPEC and POSHIST for GBM
;			each directory should contain a day of data
;			Up to 3 directories may be used to solve the intervals on the
;			day boundaries problem
;
; OPTIONAL INPUTS:
;  	ERANGE=ERANGE - energy ranges used for CSPEC rates, 6 values, default is [6,12,25,50,100,300]
;	THRESHOLD=THRESHOLD Passed to gbm_qlook_single
;	testscale=testscale Passed to gbm_qlook_single
;	SMOOTHBIN=SMOOTHBIN Passed to gbm_qlook_single
; OUTPUTS:
;       Data - a structure containing the basic rate and ephemeris info
;			** Structure <75526d8>, 9 tags, length=336, data length=333, refs=1:
		;   UTM             STRUCT    -> ANYTIM2INTS_FULL Array[1] ;Time
		;   DUT             DOUBLE           4.0960001 ;diff in sec between intervals
		;   DF              FLOAT     Array[5, 6] ; five energy ranges of rates, 6 Sunward detectors
		;					Nominal energy ranges: [6,12,25,50,100,300] keV, Counts/s
		;   DR              FLOAT     Array[5, 6] ;five energy ranges of rates, 6 A-Sunward detectors
		;   RATE            FLOAT     Array[5]	  ; df-dr, totaled over detectors
		;   HI              FLOAT      1.87337e-009 ;Higher energy GOES at UTM
		;   LO              FLOAT      3.02004e-007 ;Lower energy GOES at UTM
		;   DN              BYTE         1          ;Daynight flag, day is 1
		;   DETCOSINES      FLOAT     Array[12]     ;detector cosines to Sun at UTM
;		Synrate - a synthetic rate built from data.rate[0:1], data.hi, and data.dn
;
; OPTIONAL OUTPUTS:
;	none
;
; KEYWORDS:
;	none
; COMMON BLOCKS:
;	none
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	Meant to be called from gbm_cat to build gbm flare catalog
;	Keywords are not checked for valid ranges.  If you don't know how they
;	are used, don't change them.
;
; PROCEDURE:
;	This procedure concatenates the results of gbm_qlook_single over multiple sets of
;	daily files.
;
; MODIFICATION HISTORY:
;	18-MAR-2010, richard.schwartz@nasa.gov
;
;-

pro gbm_qlook, daily_files,  data, synrate, havedata=havedata, $
	 _extra=_extra

havedata = 0

;Check to see if daily_files are multiple directories or files for multiple days
ndays= n_elements(daily_files)
isdir = is_dir(daily_files[0])

if isdir then begin
	daily = ptrarr(ndays)
	for i=0,ndays-1 do daily[i] = ptr_new(daily_files[i])

endif else begin
	alldays = anytim(file2time(daily_files),/date)
	days  = get_uniq( alldays, count=count)
	daily = ptrarr(ndays)
	for i=0,ndays-1 do daily[i] = ptr_new(daily_files[where(alldays eq days[i])])
endelse

first = 1
for i=0,ndays-1 do begin ;ndays should be 3 or fewer

	gbm_qlook_single, *daily[i],  data1, synrate1, status=status, _extra=_extra
	if ~status then goto, next
	
	if first then begin
		data = data1
		synrate = synrate1
		first = 0
	endif else begin
			nd = n_elements(data)
			d100 = where( anytim(data[nd-100:*].utm) gt (anytim(data1[0].utm)-.01), n100)
			if n100 ge 1 then begin
				data = data[0:nd-100+d100[0]-1,*]
				synrate = synrate[0:nd-100+d100[0]-1,*]
			endif

			data = concat_struct( data, data1)
			synrate = [synrate, synrate1]
	endelse
next:	
endfor

havedata = exist(synrate) ? 1 : 0

free_var, daily

end