;+
; Name: lat_plot_lc
; 
; Purpose: Plot Lat Lightcurves corrected for exposure.  Option to plot counts in bin, or exposure data instead, and option to overplot calculated 
;   Good Time Intervals (GTI) and/or times covered by LAT Response files
; 
; Calling Sequence:
;   lat_plot_lc, date[, corrected=corrected, /exposure, /show_gti, /show_rsp, lc_data=lc_data]
;    
; Input Argument:
;   date - time interval to plot exposure for (in anytim format). 2-element range, or if scalar assumes 1 day.
;   
; Input Keywords:
;   corrected - if set, plot exposure-corrected counts (default is 1).  Otherwwise, just counts
;   exposure - if set, plot exposure instead of counts (default is 0)
;   show_gti - If set, overplot the GTI intervals (default is 0)
;   show_rsp - If set, overplot the time intervals that the RSP files were made for (default is 0)
;   desc - Name of plotman panel (if omitted will default to title of plot)
;  
; Input/Output Keywords:
;   plotman_obj - returns plotman object reference, which can be used as input for another call to lat_plot_lc
;   
; Output Keyword:
;   lc_data - returns structure containing time intervals (in sec since 1979/1/1), exposure in cm^2/s for 60 sec bin, and cts in 60sec bin.
;   
; Output: Plots the exposure-corrected counts, just counts, or just exposure information and GTI and RSP time info in a plotman instance.
;
; Method: Reads LAT daily Lightcurve (LC) file in hesperia LAT solar archive.  If user is not on hesperia computer,
;  copies file to user's local directory.  If requested time spans a day boundary concatenates data from each
;  daily file. GTI info also comes from LC file.  RSP times are found by parsing the file names of the RSP files
;  in the hesperia LAT RSP archive.
;  
; Examples:
; lat_plot_lc, '25-feb-2014'
; lat_plot_lc, ['25-feb-2014','27-feb-2014 00:00'], /show_gti, /show_rsp
; lat_plot_lc,'1-sep-2014 ' + ['10:50','11:30'], lc_data=lc_data
; 
;  
; Kim Tolbert, 29-Feb-2016
; Modifications: 
;
;-

;-------------------------------
;
; This routine will be called by the utplot object used in plotman called by lat_plot_lc (via addplot_name) to draw bars showing the 
; good time intervals (gtis) and /or the response file times (if either are available).
; 248 is green in plotman, 243 is red.
pro lat_plot_lc_marks, _extra=arg
if arg.gtis[0] ne -1 then begin
  mark_intervals, arg.gtis[0,*], arg.gtis[1,*], col=248, n_bar=n_bar, /longline
  text_leg = 'Good Time Intevals'
  col_leg = 248
endif
if arg.trsp[0] ne -1 then begin
  mark_intervals, arg.trsp[0,*], arg.trsp[1,*], col=243, n_bar=n_bar, /longline
  text_leg = append_arr(text_leg, 'Response File Times')
  col_leg = append_arr(col_leg, 243)
endif

if exist(text_leg) then ssw_legend, [' ',text_leg], textcolor= [0,col_leg], thick=3, box=0, charsize=!p.charsize*.8
end

;-----------------------------------

pro lat_plot_lc, date, corrected=corrected, exposure=exposure, show_gti=show_gti, show_rsp=show_rsp, lc_data=lc_data, plotman_obj=pl, desc=desc

checkvar, corrected, 1
exposure = keyword_set(exposure)
show_gti = keyword_set(show_gti)
show_rsp = keyword_set(show_rsp)

what = exposure ? 'Solar Exposure' : (corrected ? 'Exposure-corrected Counts' : 'Counts')
ylog = exposure ? 0 : 1
units = exposure ? 'cm!u2!n s' : (corrected ? 'counts cm!u-2!n s!u-1!n' : 'counts per bin (60s)')

lc_data = lat_get_exposures(date, gtis=gtis)
if ~is_struct(lc_data) then return

; In lc_data structure, exposure tag is in cm^2 s for 60 sec,  counts tag is counts in bin which is 60 sec
; So could divide exposure by 60 to get expos in cm^2. divide counts by expos to get cts / cm^2 in 60 sec, and then divide by 60, but
; that's the same thing as just dividing the given counts by expos (the 60s cancel out)

if ~show_gti then gtis = -1

np = n_elements(lc_data)
tplot = anytim(/vms,lc_data.times)
yplot = exposure ? lc_data.expos : (corrected ? f_div(lc_data.cts, lc_data.expos) : lc_data.cts)

tr = minmax(lc_data.times) 

if show_rsp then trsp = lat_rsp_times(date) else trsp = -1

utobj = obj_new('utplot', tplot, yplot, id='Fermi LAT ' + what, data_unit=units, ylog=ylog)
utobj -> set, addplot_name='lat_plot_lc_marks', addplot_arg={gtis: gtis, trsp: trsp}
utobj->plotman, plotman_obj=pl, desc=desc

end
