;+
; Name: messenger_obs_times
; 
; Purpose: Get time intervals when MESSENGER was observing the Sun during times covered by input files.
; This is called by messenger_obs_times_write to write a text file with the solar obs times.
; 
; Input arguement:
;  files - MESSENGER .dat files (corresponding .lbl files must be in same directory) to read and determine
;    solar observing times (have non-zero data, not in eclipse, not off, not too hot)
;    
; Output keywords:
;  count - Number of time intervals returned
;  
; Output - function returns a 2xn array of time intervals in sec since 79/1/1
; 
; Written: Kim Tolbert, Nov. 2014
; Modifications:
;-

function messenger_obs_times, files, count=count

nfiles = n_elements(files)

s = sort(file_basename(files))
sfiles = files[s]

for ii=0,nfiles-1 do begin
  print,sfiles[ii]
  read_messenger_pds,file=sfiles[ii],data_str=data_str,err_msg=err_msg,/verb
  if err_msg ne '' then begin
    print,err_msg
    continue
  endif

  ; read_messenger_pds inserts 0's for times when in eclipse, too hot, etc.
  ; if all data in entire file are 0's then skip file
  qz = where(total(data_str.rcounts,1) ne 0., nz)
  if nz eq 0 then continue  ; skip to next file
  
  ; find element number of contiguous non-zero data
  dum1 = find_contig(qz, dum2, ss_sel_2d)
  qzz = qz[ss_sel_2d]
  if n_elements(qzz) gt 2 then qzz = transpose(qzz)  ; want [2,n], not [n,2] which is what find_contig returns
  goods = reform(data_str.ut_edges[0,qzz[0,*]])
  goode = reform(data_str.ut_edges[1,qzz[1,*]])
  good_new = transpose([[goods],[goode]])
  width = get_edges(data_str.ut_edges,/width)
  
  ; good_new are the good intervals from this file.  Now merge with good intervals from previous files (good_all).
  ; If the first new interval overlaps with last interval in good_all, then extend end of last good_all 
  ; interval to include up to end of first good_new interval. Then concatenate good_all and good_new
  if ~exist(good_all) then good_all = good_new else begin
    ng = n_elements(good_all[0,*])
    if (good_new[0,0] - good_all[1,ng-1]) le (last_item(width) + .1) then begin
      good_all[1,ng-1] = good_new[1,0]
      if n_elements(good_new[0,*]) gt 1 then begin
        good_new = good_new[*,1:*]
        good_all = [[good_all],[good_new]]
      endif
    endif else good_all = [[good_all],[good_new]]
  endelse
endfor

if exist(good_all) then begin
  count = n_elements(good_all[0,*])
  return, good_all
endif else begin
  count = 0
  return, -1
endelse

end

