;+
; PROJECT:
;	SOXS
; NAME:
;	SOXS_CZT_DRM
;
; PURPOSE:
;	This function constructs and returns the DRM for SOXS czt detector
;
; CATEGORY:
;	Spectral Analysis
;
; CALLING SEQUENCE:
;	soxs_czt_drm, $
;	    fwhm=fwhm, $
;	    gain=gain, $
;	    ein2 = ein2, $
;	    area = area, $ ;nominal geometric area
;	    error = error
;
;
; KEYWORD INPUTS:
;
;		FWHM - constant, uses 2.0 keV
;
;		NCHAN - number of channels for DRM, default is 238
;		GAIN - keV per channel, default is 0.218750 from R Jain
;		AREA - default is 0.18 cm^2
;
; OUTPUTS:
;	   Returns DRM as nchan x nchan array from 4-~56 keV
;			for a photon spectrum in photons/cm2/sec input
;			(i.e. integrated over photon energy bins), the output
;			is in units of cnts/cm2/s/keV
;			count_flux = drm # (photon_spectrum * DE)
;			where photon_spectrum is in units of photons/cm2/sec/keV
;			and DE is the input energy bin width in keV
;       EIN2 - edges in 2 x nchan array
;
;	    ERROR - 0/1 means no error/error
;
;
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	Reads efficiency, transmission, and edge files provided by
;	SOXS team.  Energy calibration is fit and uses, efficiency and transmission
;	are read and smoothed to energy edge midpoints.
;
; MODIFICATION HISTORY:
;	7-dec-2006, richard.schwartz@gsfc.nasa.gov
;	21-Sep-2007, Kim.  Made offset and fwhm inputs work.
;	28-Sep-2007, Kim.  Set default offset to 2
; 20-feb-2008, richard.schwartz@nasa.gov
;	removed 3 lowest data bins on direction from R Jain. Maintained
;		channel gain for 56 keV over 256 channels(implemented in soxs_czt_drm)
;
;-


function soxs_czt_drm, $
    fwhm=fwhm, $
    gain=gain, $

    ein2 = ein2, $
    area = area, $ ;nominal geometric area
    nchan= nchan, $
    error = error

error = 1
drm = 0

default, area, 0.18 ; cm^2
default, fwhm, 2.0
default, gain, 0.21875 ;(60.-4)/256. ;original nominal gain

default, nchan, 238

ein2  = get_edges(/edges_2, 4.+findgen(nchan+1)*gain)  ;build (nchan+1) channels with this gain starting at 4 keV
ems   = get_edges(ein2,/mean)
oeres = energy_res(ein=ein2, eout=ein2, e_vs=ems, fwhm=ems*0+fwhm, sig_lim=8)
eres = oeres->getdata()
obj_destroy, oeres
czteff  = soxs_czt_photopeak( ems>2, area=area, error=error)
if error then return, 0

czteffmtrx= rebin(transpose(czteff),nchan,nchan)
drm = eres * czteffmtrx / rebin( gain+fltarr(nchan), nchan, nchan)/area
error = 0
return, drm
end
