;+
; PROJECT: RHESSI
;
; NAME: SPEX::CHI2_MAP_GRID 
;
; PURPOSE: OSPEX method to determine the errors in two best-fit parameters via chisquare mapping over a 2-D grid.
;   The selected fit parameters are fixed at a grid of values centered on the best-fit value, and
;   the remaining parameters are fit.  Contours of the chi-square at each fit are plotted on a map of the xy space
;   of param1 vs param2.
;   (usually looks like a smooth inverted gaussian), and the parameter values at the chisqr min plus 1 and
;   the chisqr min plus 4 give the 1- and 2-sigma error estimates.   
;   
; EXPLANATION: After the user sets up their ospex object, defines input file, background options, fit 
;   intervals and function, and finds the best-fit parameters, the user can call this method to
;   determine the error on two of those best-fit parameters for one of those intervals via a chisqr-mapping technique.  
;   We loop through a 2-D grid of values surrounding the best-fit value for the selected parameters, setting the parameters
;   to the coordinates of that grid point and fixing them.  The remaining parameters are fit.  This yields a chisqr value 
;   for each pair of fixed value of the selected parameters.  We save the results in an IDL save file.   We call 
;   spex_chi2_map_grid_results to compute the
;   1-sigma and 2-sigma errors based on chisqr min plus 1 and 4.
;   
;   The order of going through the grid is to start at the best fit value in param1 and go forward and backward from there
;   Then start of best-fit value in param2 and go up and down from there
;   i.e. we start with the best-fit and use the results of each fit as starting parameter (for params other than those in
;   the grid) for fit at next grid point.
;   
;   Once the save file is created, the user can call spex_chi2_map_grid_results directly and request plots of the 
;   chisqr contours on screen, png, or ps.
;      
;   On exit, the ospex object is restored to its state on entering, including resetting the spex_summ
;   parameters (i.e. the values from the last fit iteration in this routine are not what is left in the 
;   spex_summ structure for this interval upon exit).  This means that if desired, you can run this routine
;   again with the same object rather than reinitializing the object with your original best-fit params.
; 
;   RESTRICTIONS:
;   
;   This works on only one interval at a time (specify the interval in the keyword).
;   
;   Note:  A chi-square map on a single variable and monte carlo method ire also available for determining error 
;   estimates (see the spex__chi2_map and spex__monte_carlo routines).
;   Monte Carlo works well in cases where your best-fit parameters are well-defined.  It assumes
;   they are the correct answer, and varies the model around them.  It does not explore parameter space far
;   away from your best-fit parameters.  Chi-square mapping (done in this routine) or Bayesian analysis is 
;   more appropriate for cases where there are multiple possible solutions.
;   
; CALLING SEQUENCE: 
;   o -> chi2_map_grid
;  
; INPUT KEYWORDS:
;  param1_index - first parameter index to find error for (default=0)
;  param1_range - range of values for param1 grid to cover (default is best fit +- 3.*sigma)
;  param1_nvals - number of values in grid for param1 (default is 50)
;  param2_index - second parameter index to find error for (default=0)
;  param2_range - range of values for param2 grid to cover (default is best fit +- 3.*sigma)
;  param2_nvals - number of values in grid for param2 (default is 50)
;  interval - fit interval number (default is 0)
;  id - string to include in output file name if it is being auto-generated to make filename unique
;     e.g., if id='_fixedweights', the auto-generated name might be 'chi2_map_grid_i1_p2_n40_p4_n60_fixedweights.sav'
;  savefile - name of save file to write (default is chi2_map_grid_iI_pJ_nK_pL_nM.sav, where I,J,K,L,M are:
;    I=interval, J=index of param1, K=number of param1 points, L=index of param2, M=number of param2 points)
;  plot - if set, plot results in spex_chi2_map_grid_results (default is 1)
;  quiet - if set, don't print info about each iteration.  Default is 1.
;  _extra - any additional keywords to pass through to spex_chi2_map_results (e.g. /ps, /png)
;
; OUTPUT KEYWORDS:

;
; Example:
;  o->chi2_map_grid, param1_index=4, param2_index=6, param1_nvals=5, param2_nvals=7, interval=1
;  o->chi2_map_grid, param1_index=4, param2_index=6, param1_nvals=50, param2_nvals=60, param1_range=[3.,3.5], param2_range=[4.,4.7],interval=1
; OUTPUT:
;  A save file will be created containing the following values from the chisqr mapping process:
;  fit_function, interval, fit_time, param_index, savep, savechi, savefullchi, savestart
;  
; WRITTEN: Kim Tolbert, 30-Aug-2012
; 
; MODIFICATIONS:
;
;-

pro spex::chi2_map_grid, $
  param1_index=param1_index, $
  param1_range=param1_range, $
  param1_nvals = param1_nvals, $
  param2_index=param2_index, $
  param2_range=param2_range, $
  param2_nvals = param2_nvals, $  
  interval=interval, $
  id=id, $  
  savefile=savefile, $
  quiet=quiet, $
  _extra=_extra

checkvar, param1_index, 0
checkvar, param2_index, 1
checkvar, param1_nvals, 50
checkvar, param2_nvals, 50

checkvar, interval, 0
checkvar, id, ''
checkvar, savefile, 'chi2_map_grid_i' + trim(interval) + '_p' + trim(param1_index) + '_n' + trim(param1_nvals) + $
  '_p' + trim(param2_index) + '_n' + trim(param2_nvals) + id + '.sav'
checkvar, plot, 1
checkvar, quiet, 1

; initialize output variables in case we abort
out_struct = -1
vals = -1
chi2 = -1

; save original spex_summ values
summ_orig = self -> get(/spex_summ)

; Make sure requested interval # is defined and has been fit.
if interval ge n_elements(summ_orig.spex_summ_fit_done) || $
  summ_orig.spex_summ_fit_done[interval] ne 1 then begin
  message, /cont, 'Requested interval ' + trim(interval) + ' does not exist, or has not been fit.  Aborting'
  return
endif

; Get previously stored best-fit parameters and sigmas for requested interval
params = summ_orig.spex_summ_params[*,interval]
sigmas = summ_orig.spex_summ_sigmas[*,interval]
nparams = n_elements(params)

; Make sure requested parameter index is valid
if param1_index ge nparams then begin
  message, /cont, 'Requested parameter index ' + trim(param1_index) + ' does not exist.  Aborting.'
  return
endif

if param2_index ge nparams then begin
  message, /cont, 'Requested parameter index ' + trim(param2_index) + ' does not exist.  Aborting.'
  return
endif

; Get errors to use for weighting for all parameter values. (want same weighting for entire map)
errors = summ_orig.spex_summ_ct_error[*,interval]

; Save some settings so we can restore them when we're done.
nointeractive_orig = get_logenv('OSPEX_NOINTERACTIVE')
orig_vals = self -> get(/spex_fit_firstint_method, $
  /spex_fit_start_method, $
  /spex_fit_init_params_only, $
  /spex_fit_manual, $
  /spex_interval_index, $
  /fit_comp_params, $
  /fit_comp_free_mask)

; don't plot or popup widgets while running the chi2 mapping
set_logenv, 'OSPEX_NOINTERACTIVE', '1'

; Use 'current' instead of 'prev_iter' so that we can store params back into fit_comp_params for
; each iteration, and it will use them to start (otherwise would use results from most recent fit)
; Have to also set spex_fit_start_method because if it's already set to 'previous_iter', that
; overrides spex_fit_firstint_method='current'.
; Set spex fit methods to use current params, and auto-fit, and to work on selected interval
self -> set, spex_fit_firstint_method='current', $
  spex_fit_start_method='previous_int', $
  spex_fit_init_params_only=0, $
  spex_fit_manual=0, $
  spex_interval_index = interval, $
  spex_intervals_tofit = interval

; get free mask used for this interval, and set parameters we're working on to fixed.
free_mask = summ_orig.spex_summ_free_mask[*,interval]
free_mask[param1_index] = 0
free_mask[param2_index] = 0

; set min, max, spectrum, model, erange, uncert, itmax previously used for this interval
set_all=1 & set_all_comp=1
(self->get(/obj,class='spex_fit')) -> set_fitting_params,  set_all, set_all_comp, summ_orig, params, interval
self -> set, fit_comp_free_mask = free_mask

if quiet then begin
  print, ' '
  print, 'Quiet is set, so will print update only every 5 iterations.'
  print, 'For more output, call with quiet=0'
endif

fit_function = self -> get(/fit_function)
fit_time = self -> get(/spex_fit_time_interval)
fit_time = anytim(fit_time[*,interval],/vms)

; b1,b2 are best fit values for params 1 and 2
; r1,r2 are range for params 1 and 2.  If range not passed in, default is best +- 3*sigma
; n1,n2 are number of points in grid for params 1 and 2
; d1,d2 are delta in grid for params 1 and
; p1,p2 are arrays of grid values for params and 2
; s1,s2 are indices of best fit (b1,b2) in p1,p2 arrays
b1 = params[param1_index]  ; best values
b2 = params[param2_index]
checkvar, param1_range, [b1 - 3.*sigmas[param1_index], b1 + 3.*sigmas[param1_index]]
checkvar, param2_range, [b2 - 3.*sigmas[param2_index], b2 + 3.*sigmas[param2_index]]
r1 = param1_range ; range of values to cover
r2 = param2_range
n1 = param1_nvals  ; # values in grid
n2 = param2_nvals 
d1 = (r1[1] - r1[0]) / n1  ; delta
d2 = (r2[1] - r2[0]) / n2
p1 = [[b1 - d1 * indgen(n1)], [b1 + d1 + d1 * indgen(n1)]]  ; p1,p2 are array of param values
p1 = p1[where(p1 ge r1[0] and p1 le r1[1])]
p1=p1[sort(p1)]
p2 = [[b2 - d2 * indgen(n2)], [b2 + d2 + d2 * indgen(n2)]]
p2 = p2[where(p2 ge r2[0] and p2 le r2[1])]
p2=p2[sort(p2)]
s1 = value_locate(p1,b1)  ;s1,s2 are indices of best fit in p1,p2
s2 = value_locate(p2,b2)

; create arrays for storing results of fit iterations

savep = fltarr(nparams,n1,n2)
savechi = fltarr(n1,n2)
savefullchi = savechi
savestart = fltarr(nparams,n1,n2)


best_params = params
;print, 'newpar = ',newpar
;print, 'best_params = ', best_params
start_center = 1

starting = best_params

;loop through n1 values of param1 and n2 values of param2, fitting other parameters.
; Use errors from best fit so weighting doesn't change for chi2 map.
for ii = 0,n1-1 do begin
  if ii le s1 then begin
    i = s1-ii
    starting = ii eq 0 ? best_params : savep[*,i+1,s2]
  endif else begin
    i = ii
    starting = savep[*,i-1,s2]
  endelse
      
  for jj=0,n2-1 do begin
    if jj le s2 then begin
      j = s2-jj
      if jj ne 0 then starting = savep[*,i,j+1]
    endif else begin
      j = jj
      if jj ne 0 then starting = savep[*,i,j-1]
    endelse
        
    print,'ii,jj,i,j = ', ii,jj,i,j
    print,'p1[i],p2[j] = ', p1[i],p2[j]
    
    params = starting
    params[param1_index] = p1[i]
    params[param2_index] = p2[j]
    self -> set, fit_comp_param=params
    if ~quiet then begin
      print,'Start parameters = ', params
      print,'Free mask        = ', free_mask
    endif
    
    self -> dofit, use_errors=errors, quiet=quiet
      
    savep[*,i,j] = self -> get(/fit_comp_params)
    chi2 = self -> get(/mcurvefit_chi2)
    fullchi2 = self -> calc_summ(item='chisq_full') 
    savechi[i,j] = chi2
    savefullchi[i,j] = fullchi2[interval]
    savestart[*,i,j] = params
      
    if ~quiet or ((jj+1) mod 5 eq 0) then begin
      print,''
      print,' Completed chi2 mapping trial ', trim(jj+1), ' / ',trim(n2), '   Chi2,   Fullchi2  ', chi2, fullchi2[interval]
    endif 
      
  endfor
    
  if ~quiet or ((ii+1) mod 5 eq 0) then begin
    print,''
    print,' Completed chi2 mapping trial ', trim(ii+1), ' / ',trim(n1), '   Chi2,   Fullchi2  ', chi2, fullchi2[interval]
  endif 
    
  save, fit_function, interval, fit_time, param_index, savep, savechi, savefullchi, savestart, file=savefile
  print,''
  print,'Wrote IDL save file ' + savefile
  print,''
endfor


; Restore values in object that we changed

self -> set, _extra=orig_vals

fit_obj = self -> get(/obj, class='spex_fit')
fit_obj -> set, _extra=summ_orig, /this_class_only
set_logenv, 'OSPEX_NOINTERACTIVE', nointeractive_orig

; Print error ranges of each fitted parameter
;spex_chi2_map_grid_results, savefile=savefile, plot=plot, out_struct=out_struct, _extra=_extra


end
