;+
; Name: spex__grs_bk_from_save
;
; Purpose: For SMM GRS data, background for an event can be set from the data from previous and/or next day.
;   For many of the GRS events stored on hesperia in https://hesperia.gsfc.nasa.gov/smm/grs/ a background
;   was created and saved in an IDL save file.  See the routine grs_make_bk_files for more details on how
;   these background rates were computed.
;   Here, if the background save file exists, we:
;     1. Set the spex_accum_time to fd_accum_time from the save file (so number of bins will match bk bins)
;     2. Set the background time interval to anything (here we set to fd_accum_time too just for simplicity)
;     3. Accumulate the 'fake' background (want to have bk arrays set to right size and have ltime available)
;     4. Use the replacedata method to insert the background and error from the save file into the ospex object
;     5. Set spex_bk_poisson_error to 1 so that background errors will be summed in quadrature (since they're
;        really counts, not the result of a fit)
;
;  Written: Kim Tolbert 15-Apr-2019
;  Modifications:
;  18-Apr-2019, Kim.  Check local directory for background file before looking across network.
;  12-Jun-2019, Kim.  Added option to find and use HEM background save file when spex_data_sel is set to 'hem'
;
;-
pro spex::grs_bk_from_save

  specfile = file_basename(self->get(/spex_specfile))
  if specfile eq '' then begin
    message, 'No spex_specfile set. Please set input spectrum file first. Aborting.', /cont
    return
  endif

  ; The original USE file name is the last 8 characters before '.fits'. The background file corresponding to
  ; specfile will contain this string.
  usename = strmid(specfile, 12, 8, /reverse)
  
  hem = self->get(/spex_data_sel) eq 'hem' ? 1 : 0
  search_string = hem ? 'smm_grs_hem_1*' :'smm_grs_1*'

  ; First see if correct background file containing usename is on local directory
  bk_search = search_string + usename + '_background.sav'
  bkfile = file_search(bk_search,count=count)
  
  ; If didn't find it, look for it on hesperia
  if count eq 0 then begin
    ; Use smm_grs object to get host and dir name for smm grs data, can't use obj to retrieve background file though.
    ogrs = obj_new('smm_grs')
    rhost = ogrs->getprop(/rhost)
    topdir = ogrs->getprop(/topdir)
    destroy,ogrs
    year = strmid(file2time(specfile, /ecs), 0, 4)

    ; Find background file with this usename on host rhost
    rbkfile = sock_find(rhost, path=topdir + '/' + year, bk_search, count=count)
    if count eq 0 then begin
      message, 'No background file available for this input file: ' + specfile + '. Aborting.', /cont
      return
    endif

    ; Copy and restore the background file.
    ; This restores readme, fd_file_name, fd_accum_time, back_grs, and eback_grs
    sock_copy, rbkfile, local_file=bkfile
  endif

  restore, bkfile

  ; Have to set the time to use from specfile so that # bins will match back_grs, eback_grs arrays.
  self->set, spex_accum_time = fd_accum_time
  dummy = self->getdata()

  ; set the background time intervals to the total time we're using for simplicity, actual
  ; time doesn't matter, just want to be able to call getdata, and have oldbk with the right dimensions
  self->set, spex_bk_time_interval=fd_accum_time

  ; since errors are not the result of a fit, but are actual counts from different times, we want to
  ; sum them in quadrature (otherwise we average the errors) NOTE: set this before calling replacedata
  ; otherwise it resets the need_update flag to 1!!!
  self->set, spex_bk_poisson_error = 1

  oldbk = self->getdata(class='spex_bk')

  ; now insert the back and error in back into the object. They were in counts/sec, so multiply
  ; by ltime to insert as counts.
  self->replacedata, back_grs * oldbk.ltime, /bkdata, /counts
  self->replacedata,eback_grs * oldbk.ltime, /bkdata, /error

  print, 'Background from file ' + bkfile + ' was set into OSPEX object.'

end

