;+
; Name: template_select 
;
; Purpose:  OSPEX method for selecting template files for template function components
;  (same idea as nuclear_table_load for old spex)
;
; Inputs:
;  template_file - string scalar or vector containing template file names
;  func_index - integer scalar or vector containing which template component to assign
;    corresponding template_file to
;  NOTE: template_file and func_index must have same number of elements.
;  
;  Output:
;  error - 0/1 indicates failure/success
;
; Example:
;   If OSPEX fit_function is vth+template+bpow+template then
;     o->template_select, ['a.sav,'b.sav'], [0,1]
;   would force the first f_template to use a.sav, and the second f_template to use b.sav
;     o->template_select, 'c.sav', 1
;   would force second f_template to use c.sav
; History
; Kim Tolbert, 20-Sep-2009 
;
;-
pro spex::template_select, template_file, func_index, error=error

error = 0
if n_params() ne 2 || $
  (size(template_file, /tname) ne 'STRING') || $
  (size(func_index, /tname) ne 'INT') || $
  (n_elements(template_file) ne n_elements(func_index)) then begin
    error = 1
    message,/cont,'Error in calling arguments.'
    print,'Syntax:  o->nuclear_table_select, template_file, func_index'
    print,'  where template_file is string or string array, func_index is int or int array'
    print,'  and template_file and func_index must have same number of elements.'
    return
endif

func = self -> get(/fit_function)
names = self -> get(/compman_name, class='fit_comp_manager')
strats = self -> get(/compman_strategy, class='fit_comp_manager')

q = where (names eq 'template', count)
if count gt 0 then begin
  n_input = n_elements(template_file)
  for i=0,n_input-1 do begin
    index = func_index[i]
    if index gt (count-1) then begin
      message,/cont, 'No f_template component for template_file ' + template_file[i]
      return
    endif
    print,'Setting ' + strats[q[index]] + ' to file ' + template_file[i]
    self -> set, fit_comp_spectrum=template_file[i], strategy_name=strats[q[index]]
  endfor
endif

end

