;+
; Name: spex_bk_profile
; 
; Purpose: Compute smoothed, interpolated profile based on specified intervals of input profile.  
; Start with input time profile, smooth  selected time intervals with boxcar of width 10 (so that endpoints we use 
; for interpolation are good), interpolate across gaps in specified intervals, then smooth again using savgol filter 
; of width sm_width.  
; Note that we're doing this in rate space. 
; Currently used to compute profile of background data for use in the two ratio-to-profile background methods in OSPEX
;
; Input Keywords:
;  data - input time profile of data in rate units
;  tmid - mid-points of times corresponding to data in sec since 1979
;  sel - indices in data array to use to create profile
;  sm_width - savgol half smoothing width (0 means don't smooth, 1 means use 3 points, 2 means use 5 points, etc.)
;  plot - if set, plot the input, smoothed, interpolated, and final smoothed profile
;  eband - the energy band covered by the input data (2 element array of low, high keV)
; 
; Output Keywords:
;  err_msg - string containing any error messages
;  
; Written: Kim Tolbert 17-Sep-2013.  (Extracted this code from spex_bk::process, and modified to be standalone)
; Modifications:
; 
;-

function spex_bk_profile, data=data, tmid=tmid, sel=sel, $
  sm_width=sm_width, plot=do_plot, eband=eband, err_msg=err_msg

checkvar, do_plot, 0
eband_s = keyword_set(eband) ? format_intervals(eband) + ' keV' : 'high energy band'
err_msg = ''

profile = data
orig_profile = profile

if sel[0] eq -1 then begin
   err_msg = 'No time intervals selected for ' + eband_s + '.'
   goto, error_exit
endif
q = where (profile[sel] gt 0.,count)
if count gt 0 then sel = sel[q] else begin
   err_msg = 'No good data in ' + eband_s + '.'
   goto, error_exit
endelse

; Find groups of contiguous points in the background intervals selected and within each group, smooth
;  the data with a boxcar width of 10 (or fewer if there are < 10 points) so that a high or low endpoint
;  doesn't throw off the interpolation between the groups of points.
;  Then for the first two points in the first group and the last two points in the last group, replace those
;  two points with the average of the first (or last) up to 10 points (if available), so the
;  extrapolation to the earliest and latest times is flat and doesn't go off in an arbitrary
;  direction. 
res = find_contig(sel, ss, q_2d) ; q_2d will be a [n,2] where n is the group number, 2 is for start/ind
ngroups = n_elements(ss)
q_2d = reform(q_2d, ngroups, 2)  ; in case ngroups is 1, want [1,2] array, not just [2]
for k=0,ngroups-1 do begin
  ind = sel[q_2d[k,0]:q_2d[k,1]]
  i1 = ind[0] & i2 = last_item(ind) & nind = n_elements(ind)
  profile[ind] = smooth(profile[ind], 10<(nind-1), /edge_truncate)
  if k eq 0 && nind gt 1 && i1 ne 0  then $
    profile[ind[0:1]] = average(profile[ind[0:9<(nind-1)]])
  if k eq ngroups-1 && nind gt 1 && i2 ne n_elements(profile)-1 then $
    profile[ind[nind-2:nind-1]] = average(profile[ind[(nind-10)>0:*]])
endfor
profile2 = profile

base_time = min(tmid)
profile = interpol(profile[sel], tmid[sel]-base_time, tmid-base_time)
profile3 = profile

; Now that we've done interpolation using the smoothed data, insert the original data into the
; non-interpolated bins.
profile[sel] = orig_profile[sel]

good = where (profile gt 0, count)

if count eq 0 then begin
  err_msg =  'Profile for ' + eband_s + ' has no points > 0.'
  goto, error_exit
endif

; Do final savgol smoothing on interpolated profile
smw = sm_width < (n_elements(good)-1)/2
degree = smw gt 3 ? 3 : 2
if smw gt 0 then begin 
  profile[good] = convol (profile[good], savgol(smw, smw, 0, degree), /edge_truncate)
endif

; If requested, plot each step of preparing profile
if do_plot eq 1 then begin
  device,get_screen=s
  window, /free, xsize=s[1], ysize=s[1]*.6        
  tvlct,r,g,b,/get & linecolors
  tme = anytim(tmid, /ext)
  q = where(orig_profile gt 1.e-10, nq)
  if nq gt 0 then begin    
    utplot,tme,orig_profile,/ynozero,ytit='Rate',title='Profile for '+eband_s,charsize=1.3, /ylog, yrange=minmax(orig_profile[q])
    outplot,tme[*,sel],profile2[sel],col=2,psym=2 & outplot,tme,profile3,col=5 & outplot,tme,profile,col=7
    ssw_legend, ['Original','Prepared for interp','After interp','Final'], textcol=[255,2,5,7]
  endif
  tvlct,r,g,b
endif

return, profile
   
error_exit:
return, -1
end      