;+
;
; NAME:
;	SPEX_GRS_RESP
;
; PURPOSE:
;	This procedure computes the X-ray response of the two x-ray detectors included
;	in the SMM GRS instrument.
;	Depending on the value of the environment variable GRS_MATRIX_OLD returns either
;	the old grsmatrix that's been online in SSW for 30 years, or Gerry Share's
;	new version of grsmatrix
;	setenv,'GRS_MATRIX_OLD=1' to use the old one
;	The default is to use the G. Share's new matrix
;
; CATEGORY:
;	SMM, GRS
;
; CALLING SEQUENCE:
;	SPEX_GRS_RESP,  EDGES_OUT=EDGES_OUT, EDGES_IN=EDGES_IN, $
;		AREA=AREA, DRM=DRM
;
; CALLS: LOC_FILE, CURDIR
;
;
; KEYWORDS:
;	EDGES_OUT- Energy channel edges in keV.
;	  NOTE: (for new matrix, need to supply this, for old matrix it is returned)
;	EDGES_IN - Photon input energies for DRM in keV.
;	DRM - detector response, counts/sec/cm2/keV per photon/keV/cm2.
;	AREA - 317 cm^2
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;       none explicit, only through commons;
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	none
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	The files, drm_mass_brd.txt, en_wid.txt, and grsmatrix.dat must exist in $SSW_OSPEX/smm_grsresp or the current directory.
;
; PROCEDURE:
;	The response function in grsmatrix.dat was compiled from the GRS ANATOT procedure.
;
; MODIFICATION HISTORY:
;	Written: Kim Tolbert 20-Mar-2019
;	  copied from grs_resp in the SSSW smm grs branch and modified to have the option
;	  to use the old response matrix generated by the GRS ANATOT procedure or the
;	  new response matrix generated by Gerry Share / Richard Schwartz. (uses new by default)
;	Modifications:
;	25-Mar-2019, Kim. The en_wid.txt file is not written with enough precision, resulting in
;	 photon energy bins that are not contiguous and overlap.  Compute them differently so they
;	 are contiguous. (Some of ospex fit functions require contiguous energies)
;	27-Mar-2019, Kim. Richard modified G. Share's matrix to be centered on the bin centers, so read
;	  the new save file that now has ein, eout, and drmnew stored.
;	09-Apr-2019, Kim. Bin widths we divide matrix by should be in keV, not MeV, for both old and new matrix
;	19-Apr-2019, Kim. Added files input keyword. If file name contains 'bksub_egrouped' then the drm
;	  is read from extension 3 of specfile.
;-

pro spex_grs_resp, edges_out=eout, edges_in=ein, area=area, drm=grsmatrix, files=files, error=error

  error=1

  area = 317.55 ;cm2

  ; For ...bksub_egrouped... files, drm matrix and edges are in 3rd extension of specfile
  drm_in_specfile = strpos(files[0], 'bksub_egrouped') ne -1

  if drm_in_specfile then use_new = 1 else begin
    use_old = chklog('GRS_MATRIX_OLD')
    use_new = ~(strlowcase(use_old) eq 'true' or fix(use_old) eq 1)
  endelse

  if use_new then begin
    ; Either drm is in an extension of the specfile, or we're reading it from the save file
    if drm_in_specfile then begin
      drm_struct = mrdfits(files[0], 3)
      drmnew = drm_struct.drm
      eout = drm_struct.eout
      ein = drm_struct.ein
    endif else begin
      file = loc_file('gerry_grs_drmnew.sav',path=[curdir(),'SSW_OSPEX/smm_grsresp'], count=count)
      if count eq 0 then return
      restore, file  ; restores drmnew, eout, ein
    endelse
    error = 0
    eout_wid = get_edges(eout, /wid) ; in keV since in OSPEX, we'll be multiplying by e in keV
    nout = n_elements(eout_wid)
    nin = n_elements(ein[0,*])
    grsmatrix = temporary( f_div(drmnew, rebin(eout_wid,nout,nin)) / area)

  endif else begin

    file = loc_file('grsmatrix.dat',path=[curdir(),'SSW_OSPEX/smm_grsresp'], count=count)
    if count eq 1 then begin
      error=0
      restore,file
      edge_products, [ematrix[0]-dmatrix[0]/2, ematrix+dmatrix/2]*1e3, edges_2=ein
      edge_products, [em[0]-wmatrix[0]/2, em+wmatrix/2]*1e3, edges_2=eout
      nout = n_elements(em)
      nin = n_elements(ematrix)
      ; wmatrix is bin widths in Mev, convert to keV
      grsmatrix= temporary( f_div(grsmatrix, rebin(wmatrix*1000.,nout,nin)) / area)
    endif

  endelse

end
