;+
; Name: spex_remote_template_widget
;
; Purpose: Widget to allow user to navigate the archive of gamma-ray templates stored on a server (hesperia for now)
;   select one, and download it for use in OSPEX.  When user selects a directory or subdirectory, sock_dir is called
;   to retrieve the subdirectories or files under that choice.  Once a file is selected, user clicks Retrieve to copy
;   the file to the local temp directory (whatever get_temp_dir() returns). 'Retrieve' button keeps widget open to
;   download more files, 'Retrieve and Close' retrieves file and closes widget.
;
;   This can be called standalone:
;     file = spex_remote_template_widget()
;
;   And is called from within the OSPEX Fit Component widget to select and download a template file.
;
; Written: 09/06/2021  Kim Tolbert
; Modifications:
;   23-Sep-2021, Kim.  In file list only show .sav files (each dir has a .dat file - don't want that).  Also
;     don't offset this widget from fit_comp widget, and don't offset help widget from this widget.
;   29-Sep-2021, Kim. Don't include button for just 'Retrieve .sav file' unless we're in standalong mode. 
;     Also added check for whether hesperia template dir is reachable. Returns '' if not.
;   14-Feb-2024, Kim. Made changes to accomodate new 2223_neutr_cap directory, which only has multiple 
;     .dat files, no .sav files. (previously each dir with .sav files had a single .dat file)
;
;-
pro spex_remote_template_widget_update, state

  w = state.widgets

  topdir = widget_info(w.wtd, /combobox_gettext)

  ; If topdir has changed, change all widgets below that
  if topdir ne state.td then begin
    state.td = topdir
    widget_control, /hourglass
    sock_dir, state.urlbase+topdir, list
    sd1_all = file_basename(list)
    *(state.sd1_all) = sd1_all
    widget_control, w.wsd1, set_value=sd1_all
    widget_control, w.wsd2, set_value=' '
    widget_control, w.wfile, set_value=' '
  endif

  ;  sd1_index = widget_info(w.wsd1, /droplist_select)
  ;  sd1 = (*(state.sd1_all))[sd1_index]
  sd1 = widget_info(w.wsd1, /combobox_gettext)

  ;If subdir1 has changed, then change widgets below that
  if sd1 ne state.sd1 then begin
    state.sd1 = sd1
    widget_control, /hourglass
    sock_dir, state.urlbase+topdir + '/' + sd1, list
    list = file_basename(list)
    ; See if files under sd1 are the .sav (or .dat) files or another subdirectory?
    ; Most bottom level dirs have a bunch of .sav files and one .dat file.  Exception is 2223_neutr_cap
    ;   which has no .savs file, and a bunch of .dat files.
    quse = where(strpos(list, '.sav') ne -1, nuse)
    if nuse eq 0 then quse = where(strpos(list, '.dat') ne -1, nuse)
    if nuse gt 0 then begin
      ;They're sav or dat files, so set sd2 to 'none' and list in file droplist widget
      file_all = list[quse]
      *(state.sd2_all) = 'None'
      *(state.file_all) = file_all
      widget_control, w.wsd2, set_value='None'
      widget_control, w.wfile, set_value=file_all
    endif else begin
      ;They're not sav files, they're another level of subdirectories, so fill in sd2
      sd2_all = list
      *(state.sd2_all) = sd2_all
      *(state.file_all) = ''
      widget_control, w.wsd2, set_value=sd2_all
      widget_control, w.wfile, set_value=' '
    endelse
  endif

  ;  sd2_index = widget_info(w.wsd2, /droplist_select)
  ;  sd2 = (*(state.sd2_all))[sd2_index]
  sd2 = widget_info(w.wsd2, /combobox_gettext)

  if (sd2 ne 'None') and (sd2 ne state.sd2) then begin
    state.sd2 = sd2
    widget_control, /hourglass
    sock_dir, state.urlbase + topdir + '/' + sd1 + '/' + sd2, list
    list = file_basename(list)
    qsav = where(strpos(list, '.sav') ne -1, nsav)
    if nsav eq 0 then stop,'This should not happen. Another level of subdirectory needs to be added to code.'
    file_all = list[qsav]
    *(state.file_all) = file_all
    widget_control, w.wfile, set_value=file_all
  endif

  setname:

  if (*(state.file_all))[0] eq '' then begin
    widget_control, w.wfile_text, set_value=''
    state.remote_filename = ''
  endif else begin
    ;    sd1_index = widget_info(w.wsd1, /droplist_select)
    ;    sd1 = (*(state.sd1_all))[sd1_index]
    ;    sd2_index = widget_info(w.wsd2, /droplist_select)
    ;    sd2 = (*(state.sd2_all))[sd2_index]
    ;    sd1 = state.sd1
    ;    sd2 = state.sd2
    sd1 = widget_info(w.wsd1, /combobox_gettext)
    sd2 = widget_info(w.wsd2, /combobox_gettext)
    ;    file_index = widget_info(w.wfile, /droplist_select)
    ;    file = (*(state.file_all))[file_index]
    file = widget_info(w.wfile, /combobox_gettext)
    state.file = file
    rem_file = topdir + '/' + sd1 + '/' + (sd2 ne 'None' ? sd2 + '/' : '') + file
    widget_control, w.wfile_text, set_value=rem_file
    state.remote_filename = rem_file
  endelse

  widget_control, w.wtopbase, set_uvalue=state

end

pro spex_remote_template_widget_event, event

  widget_control, event.top, get_uvalue=state

  widget_control, event.id, get_uvalue=uvalue

  if uvalue eq 'retrieve_sav_close' then begin
    uvalue='retrieve_sav'
    exit = 1
  endif else exit = 0

  case uvalue of

    'help': begin
      file = concat_dir(local_name('$SSW_OSPEX/'), 'gamma_ray_templates_help.txt')
      xdisplayfile, file, done_button='Close', group=state.widgets.wtopbase,  $
        width=130, height=50, title='Gamma-ray Templates Help'
    end

    'retrieve_dat': begin
      ; if selected filename is a .dat that means we're in the 2223_neutr_cap directories where there are 
      ; multiple .dat files. copy the selected one.  Otherwise, the directories have a single .dat file, 
      ; go find it (from current directory selections) and copy it.
      is_dat = strpos(state.remote_filename, '.dat') ne -1
      if is_dat then begin
        remote_fullname = state.urlbase + state.remote_filename
        count = 1
      endif else begin
        remote_dir = file_dirname(state.urlbase + state.remote_filename)
        remote_fullname = sock_find(str_replace(remote_dir,'\','/'), '*.dat', count=count) ; need unix style slashes
      endelse
      if count gt 0 then begin
        sock_copy, remote_fullname[0], out_dir=get_temp_dir(), local_file=local_name
        message, /info, 'Retrieved file ' + remote_fullname[0]
        message, /info, 'Copied to      ' + local_name
      endif else message, /info, 'Directory does not contain a .dat file'
    end

    'retrieve_sav': begin
      ; if we're in the 2223_neutr_cap directory, there are no save files, but multiple dat files
      ; if a dat file is selected, just print message and return. If sav file is selected, copy it
      ; and exit if requested.
      is_dat = strpos(state.remote_filename, '.dat') ne -1
      if is_dat then begin
        message, /info, 'This directory has no .sav files.
        exit=0
      endif else begin
        remote_fullname = state.urlbase + state.remote_filename
        sock_copy, remote_fullname, out_dir=get_temp_dir(), local_file=local_name
        *(state.local_filename) = local_name
        message, /info, 'Retrieved file ' + remote_fullname
        message, /info, 'Copied to      ' + local_name
        widget_control, event.top, set_uvalue=state
      endelse
    end

    'cancel': begin
      *(state.local_filename) = ''
      exit = 1
    end

    else: begin
      spex_remote_template_widget_update, state
      return
    end

  endcase

  if exit then widget_control, event.top, /destroy

end

function spex_remote_template_widget, group=group

  urlbase = 'https://hesperia.gsfc.nasa.gov/rhessi_extras/gamma_ray_templates/'
  
  sock_dir, urlbase, list
  
  if list[0] eq '' then begin
    message, /info, 'Hesperia template directory is not reachable currently.  Please try again later, or contact kim.tolbert@nasa.gov.'
    return, ''
  endif

  topdir = file_basename(list)   ; this should be ['3comp', 'abun', 'pions', '22scat']

  ; Need to use model because when called from OSPEX, this is called from a blocking widget. Can't have two
  ; layers of blocking widgets.  But caller needs to wait until this widget returns.  Modal makes this block too, 
  ; so user has to select a file and return, and then the caller has the name of the file the user selected (or 
  ; user can cancel).  Modal is only set when called with a group (parent widget that's alive).
  modal = xalive(group)
  wtopbase = widget_base(group_leader=group, /column, ypad=15, title='Gamma-ray Template Selector', /frame, modal=modal)

  wlabelbase = widget_base(wtopbase, /column, ypad=10, /align_center)
  dummy = widget_label(wlabelbase, value='Gamma-ray Template Selector')
  dummy = widget_label(wlabelbase, value='Make a selection to show options under that directory')

  wbase = widget_base(wtopbase, /column, ypad=5, space=5, /frame, /align_center)

  xsize = 250

  tmp = widget_base(wbase, /row, space=5)
  tmp2 = widget_label(tmp, value='Top Directory:  ')
  wtd = widget_combobox(tmp,  value=topdir, uvalue='topdir', xsize=xsize)

  tmp = widget_base(wbase, /row, space=5)
  tmp2 = widget_label(tmp, value='Subdirectory 1:  ')
  wsd1 = widget_combobox(tmp, value=' ', uvalue='subdir1', xsize=xsize)

  tmp = widget_base(wbase, /row, space=5)
  tmp2 = widget_label(tmp, value='Subdirectory 2:  ')
  wsd2 = widget_combobox(tmp, value=' ', uvalue='subdir2', xsize=xsize)

  tmp = widget_base(wbase, /row, space=5)
  tmp2 = widget_label(tmp, value='File Name:     ')
  wfile = widget_combobox(tmp, value=' ', uvalue='file', xsize=xsize)

  wtextbase = widget_base(wtopbase, /column, ypad=20, /align_center)
  tmp = widget_label(wtextbase, value='File name to retrieve from server:')
  wtextbase2 = widget_base(wtextbase, /column, /frame, ypad=10)
  wfile_text = widget_label(wtextbase2, value='', /align_center, xsize=450)

  wbuttonbase = widget_base(wtopbase, /row, /align_center, ypad=5, space=15)
  tmp = widget_button(wbuttonbase, value='Help', uvalue='help')
  tmp = widget_button(wbuttonbase, value='Retrieve .dat File', uvalue='retrieve_dat')
  ; Only put this button on when this widget is run stand-alone, to copy multiple files without returning.
  if ~modal then tmp = widget_button(wbuttonbase, value='Retrieve File', uvalue='retrieve_sav')
  tmp = widget_button(wbuttonbase, value='Retrieve .sav File and Close', uvalue='retrieve_sav_close' )
  tmp = widget_button(wbuttonbase, value='Cancel', uvalue='cancel')

  widgets = {wtopbase: wtopbase, $
    wtd: wtd, $
    wsd1: wsd1, $
    wsd2: wsd2, $
    wfile: wfile, $
    wfile_text: wfile_text }

  state = {td_all: topdir, $
    sd1_all: ptr_new(''), $
    sd2_all: ptr_new('None'), $
    file_all: ptr_new(''), $
    td: '', $
    sd1: 'dummy', $
    sd2: 'dummy', $
    file: 'dummy', $
    widgets: widgets, $
    urlbase: urlbase, $
    remote_filename: '', $
    local_filename: ptr_new(/alloc) }

  widget_control, wtopbase, set_uvalue=state
  spex_remote_template_widget_update, state

  ; removed offseting this widget - instead it sits right on top of fit_comp widget
  ; that way there's room for the help widget.
  ;  widget_offset, group, newbase=wtopbase, xoffset, yoffset
  ;  widget_control, wtopbase, xoffset=xoffset, yoffset=yoffset

  widget_control, wtopbase, /realize

  xmanager, 'spex_remote_template_widget', wtopbase; , /no_block

  return, *(state.local_filename)
end



