;+
; NAME:
;	Apar_init
; PURPOSE:
; 	Obtain initial values for apar; the spectral fit parameters
; CALLING SEQUENCE:
;	apar=Apar_init(e_out, counts, sigma, e_in, w_in, drm)
; INPUT:
;	f_model = Spectral model used, e.g. power law or thermal brem.
;                 e.g., 'f_bpow', 'f_vth_bpow'
;	e_out = 2xN output energy bins, ie, for the channels
;	counts = observed count rate, N numbers in the channels given by e_out
;	sigma = sigma(counts) 
;	e_in = 2xM input energy bins
;	w_in = Width of input energy bins.
;	drm  = Detector response matrix dimensioned n x m where n is the number
;	       of output energy bins and m is the number of input energy bins.
; OUTPUT:
; 	apar=initial values of a, the fit parameters
; HISTORY:
;       16-nov-95, from AINIT.pro, jmm
;-
FUNCTION Apar_init, e_out, counts, sigma, e_in, w_in, drm
   
@function_com

;first you must get the conversion factors, for a single PL, with index of 3,
;but it doesn't look like that exists, so i'll use f_bpow, also, you need to do
;this at the channel midpoints, not the edges, where e_in, and e_out are defined 
   einb = reform(e_in(0, *)+e_in(1, *))/2.0
   eoutb = reform(e_out(0, *)+e_out(1, *))/2.0
   get_conversion, e_in = einb, w_in = w_in, drm = drm, f_model = 'F_BPOW', $
     apar = [1.0, 3.0, 100.0, 3.0, 10.0, 1.5], e_out = eoutb, conversion = conversion

;now get the photon flux for each channel, the conversion factors are counts/photon
   okch = where(counts GT 0.0)
   IF(okch(0) EQ -1) THEN BEGIN
      message, /info, 'NO GOOD CHANNELS'
      RETURN, fltarr(6)
   ENDIF
   
   phflux = alog10(counts(okch)/conversion(okch))
   e_log = alog10(eoutb(okch))        ;we'll use log-log fits
   h = alog(10.0)
   sphflux2 = (sigma(okch)/(counts(okch)*h))^2 ;squared unc. in log10
   
   f_mod1 = strlowcase(strcompress(f_model, /remove_all)); to be sure
   mod_name_len = strlen(f_mod1)
   ai_model = 'ai'+strmid(f_mod1, 1, mod_name_len-1)
   apar = call_function(ai_model, phflux, sphflux2, e_log)
   
   RETURN, apar
END

