;+
;
; NAME: Intervals
;
;
; PURPOSE: Graphically select a set of intervals from an X array.
;	   The X, Y plot is already displayed
;
; CATEGORY: Graphics, Fitting
;
;
; CALLING SEQUENCE: 
;	Intervals(xdisplay, xselect ,xwidth=xwidth [,xinput=xinput] [,iselect=iselect] $
;		 [,style=style] [,color=color])
;                
;; CALLS TO: point
;
; INPUTS:
;       xdisplay - array of displayed abscissae
;		if xdisplay is 2xn, then the two vectors are assumed
;		to be the hi and lo edge of the data point, and 
;		xwidth is calculated
;	xwidth   - the width of each x bin, xdisplay taken at the center
; Optional inputs:
;       style    - options
;		 'contiguous'  The intervals are all contiguous.
;		 'discrete'    The interval boundaries are chosen individually
;		 'regular'     Two boundaries are chosen.  Intervals
;			       are chosen for all bins between boundaries
;                 Not used on direct input
;
;       xinput   - for direct input of x edges, non-graphic
;			       Non-graphic selection, a vector of 2xN time bins
;			       are passed.  No display feedback.  For input times, 
;			       the discrete mode is used although if a boundary is within 1% of an 
;			       interval width it is chosen even if it violates the leading 
;			       trailing edge rules for discrete selection
;			       
;       color    - can be used with routine linecolors
;
; OUTPUTS:
;	xselect - the value of the boundaries for each interval selected 
;		  2 x number of intervals
;	iselect - the index of each boundary relative to xdisplay
;		  for j=iselect(0,i) and k=iselect(1,i) the channels are
;		  j through k-1, inclusive
;
; RESTRICTIONS: If selected points are not within the elements of xdisplay,
;	then the element of xdisplay less than the selected value is chosen.
;	Returns 0 if called by with device set to non-interactive and graphic
;	input is required.
;
; PROCEDURE: Calls for a set of points to be input graphically or passed.  For graphical
;	selection mode, lines are
;	drawn showing each bin selected. Intervals may be chosen contiguously
;	or individually.  Default is individual points. When specifying points
;	interactively, the start and end of an interval defaults to the leading edge of
;	the selected bin for a stream of contiguous intervals, and to the trailing edge
;	for the last bin selected.
;
; MODIFICATION HISTORY:
;	ras, 4 Oct 93
;	ras, 12-feb-94, enable non-graphic selection
;-

pro Intervals, xdisplay, xselect, xwidth=xwidth ,xinput=xinput, $
	iselect=iselect, style=style, color=color
;                
; Initialize
;
xselect = [0]
iselect = [0]
dim_xdisplay = (size(xdisplay))(0)
if dim_xdisplay eq 2 then edge_products, xdisplay, width=xwidth, mean=xm $
	else xm = xdisplay

xwid = xwidth
if n_elements(xwidth) eq 1 then xwid = xdisplay*0 +xwidth(0)
edisplay = xm(*) -  xwid(*)/2. ;lower edges in display
nchan = n_elements(xm)
edisplay = [edisplay, xm(nchan-1)+xwid(nchan-1)/2] 

checkvar, color, !p.color 

if keyword_set(xinput) then style = 'I'

if !d.name eq 'REGIS' or !d.name eq 'PS' and style ne 'I' then begin
   print,'Graphic selection of intervals not available for '+!d.name+  ' graphics.'
   xselect = 0
   return
endif
error = 0

sty = strmid(strupcase(style),0,1)
                case sty of
                        'C': style = 'contiguous'
                        'D': style = 'discrete'
                        'R': style = 'regular'
			'I': style = 'inputvector'
		else:  error=1
                endcase
if error then begin
	Print,'Error in style selection, C, D, R, or I'
	return
endif
   
;
;Get boundaries
;
case style of
	'contiguous': $
		newlabel = ['Select contiguous edges, MB1 or "a-z",', $ 
			' wait for the line, MB3 or "F" to end.'] 
	'discrete': $
		newlabel = ['Select discrete edges, MB1 or "a-z",' , $
			' wait for the line, MB3 or "F" to end.']
	'regular':
	'inputvector':
endcase
select:	
	if style eq 'inputvector' then begin
		for i=0,n_elements(xinput(0,*)) -1 do begin
		  for j=0,1 do begin
	        	ix = (where( edisplay - .01*xwid gt xinput(j,i), nx))(0) -1> 0
			if j then ix=ix-1
			iselect = [iselect,ix]
		  endfor
		endfor
		goto, FINISH
	endif
					
	if style eq 'regular' then begin
		case n_elements(iselect) of
		1: newlabel = ['Select start boundary, MB1 or "a-z",' , $
			' wait for the line. ' ]
		2: newlabel = ['Select final boundary, MB1 or "a-z",' , $
			' wait for the line. ' ]
                3: goto, FINISH
		endcase
	endif

	alpha_page
	print, newlabel
	graphics_page

        if style eq 'regular' and n_elements(iselect) eq 2 then $
	  point, x, /nocross,npoint=1, newlabel='!c!c'+ $
		newlabel(0)+'!c'+newlabel(1) $
	else $
	  point, x, /nocross,npoint=1, newlabel=newlabel(0)+'!c'+newlabel(1)

	if datatype(x) eq 'STR' then goto, FINISH
        ix = (where( edisplay gt x(0) , nx))(0) -1> 0
	iselect = [iselect,ix]
	oplot, x(0) + fltarr(2), crange('y'), linest=1, color=color

	goto, select

finish:
	nselect = n_elements(iselect)-1
	if nselect le 1 then begin
		print, 'No intervals selected'
		xselect = 0
		return
	endif

	iselect = iselect(1:*)     ;Get rid of the leading zero
        if style eq 'regular' then begin
		lo = iselect(0)+indgen(iselect(1)-iselect(0)+1)
		hi = lo 
		iselect = transpose( reform( [lo,hi], n_elements(lo),2))
	endif
	if style eq 'contiguous' then begin
		lo = iselect(0:nselect-2)
		hi = iselect(1:nselect-1)		
		iselect = transpose( reform( [lo,hi-1], nselect-1,2))
		iselect(1,nselect-2) = iselect(1,nselect-2) + 1 ;choose the trailing edge of the last int.
	endif 
		nselect = n_elements(iselect)
		if nselect mod 2 ne 0 then begin
			print, 'Select the last boundary'
			goto, select
		endif
		lo = iselect( indgen(nselect/2)*2)
		hi = iselect( indgen(nselect/2)*2+1)
		iselect = transpose( reform( [lo,hi+1],n_elements(lo),2))
	
		
        xselect =  [ xm(iselect(0,*))-xwid(iselect(0,*))/2, $
        		xm(iselect(1,*))-xwid(iselect(1,*))/2 ]

return
end
	
