;+
; PROJECT: SDAC
;
; NAME:  OP_COM
;
; CATEGORY: SPEX
;
; PURPOSE:
;	This procedure handles operator communications with a menu, 
;	commands, and parameters.
; EXPLANATION:
;
;	Displays parameters and command options.
;	User responds with COMMAND or PARAMETER,VALUE
;	Valid commands are passed back to the calling program.
;	Valid parameter changes are entered into the PARAMETER structure.
;	Index of the changed parameter is passed back to the calling program
;
;	A list of commands may also be passed parsed with the command-list-delimiter
; 	This list is kept in a stack and used on subsequent calls to op_com until it
;	is finished.
;
;	Lists full menu by default.  If /NOLIST is passed, then just requests
;	command or parameter change, doesn't list.
;
;	Parameter structures should be created by calls to create_param.
;
; CALLING SEQUENCE:
; 	op_com, prm, prm_info, commands, title=title, $
;	prompt=prompt, command_prompt=command_prompt, com_delim=com_delim, $
;	families=families, nolist=nolist, input_line=input_line,
;	com_line=com_line, com_sel=command, com_param=com_param, $
;	which_par=ipar, noinputread=noinputread, error=error
;
; INPUT ARGUMENTS: 
;	PRM 	- structure with parameter values, type, etc.
;		(See create_param for info about prm and prm_info)
;	PRM_INFO- structure with information about each family of parameters
;	COMMANDS- list of allowed commands
;
; KEYWORDS:
;	TITLE         - title for menu
;	PROMPT        - prompt string on command line. default is ":"
;	COMMAND_PROMPT- optional command request string
;	COM_DELIM     - Delimiter parsing consecutive command strings, default '!!'
;	FAMILIES      - Families of parameters to display in menu.  If omitted,
;			all families in parameter structure are displayed. 
;	                All parameters in structure can still be modified, 
;	                regardless of which are displayed.
;	NOLIST 	      - If set, parameter menu is not listed
;	INPUT_LINE    - String passed in to be treated as though it had been entered
;		 	interactively on the command line.
;	COM_LINE      - Command line as entered by user (with case, spaces intact)
;	COM_SEL	      - command the user selected
;	COM_PARAM     - additional parameter modifying command
;		        command string must be different from all parameter names
;	WHICH_PAR     - index of the changed parameter
;	NOREADINPUT   - set when commands are generated external to op_com as in
;			using SPEX as a subroutine when driven by a GUI
;		        command is returned as an 'EXIT' when the stack is empty
;	ERROR	      - If set, then there was an error in the command process
;
; Below is an example of a menu, and a user's input:
; 
;	Change options by entering item, comma, new value.  After setting options,
;	enter one of the available commands.  Parameters and commands may be
;	abbreviated to shortest unambiguous abbreviation and may be strung together
;	by inserting !! between entries.
; 
;         Matrix Response Parameters
; 
;         DET_NAME       GRS_X2       
;         AREA_CM2       7.94226      
;         NAME_FUNCTION  SIGMA        
;         Z_WINDOW       1.00000      2.00000      3.00000      4.00000
;         NUM_BINS       100          
; 
;	COMMANDS:  COMPUTE LIST EXIT
; 
;	Change parameters or enter a command
; 	z_w,1.,3,4	Sets parameter z_window to 1.,3.,4.,4.
;	z_w,,,,8.	Sets fourth value in parameter z_window array to 8.
; 	(Notice multiple comma syntax.)
;	det,bat!!num,200!!z_win,1,1,1,1    Sets three parameters
;
;NON-STANDARD PROCEDURES:
;	CHECKVAR
;	PARSE_COMLINE
;	LIST_OP
;	
; programmer- ras, 92/03/26
;	      ras, 13-feb-94	include command line prompt, and blank comma syntax
;	      ras, 16-feb-94    enable command modifiers, com_param
;	Major modifications 9-mar-1994 by akt.  Mainly to use prm and fam_info
;	structures created by create_param, and perform a variety of error 
;	checks.
;	ras, 12-apr-94, made compatible with version 3.0 of READ, no
;	prompt keyword, only as inline parameter
;	ras, 16 aug 1996, clear odd blanks from com_stack
;	Version 6, ras, 28 oct 1996, send command lines to printx with /noprint
;	Version 7, ras, 24-jan-1996, trap naked PRINT, HELP, and XDOC commands
;	 and reprocess them as part of the IDL command, works if IDL command
;	is included in the calling programs command processor.
;	Version 8, richard.schwartz@gsfc.nasa.gov, allow expressions to be trapped
;	and have idl pasted onto them and executed, 6-oct-1997 also
;	trap CHKARG, SPAWN.
;-


pro op_com, prm, fam_info, title=title, $
	prompt=prompt, command_prompt=command_prompt, com_delim=com_delim, $
	families=families, nolist=nolist, input_line=input_line, $
	com_line=com_line, com_sel=command, com_param=com_param, $
	which_par=ipar, noreadinput=noreadinput, error=error
	

common op_com_com, com_stack

;more, ['!!!!!!!!!!COM_STACK!!!!!!!!!!!',com_stack]

checkvar, prompt, ':'	;standard IDL read prompt is default
checkvar, command_prompt,'Change parameters or enter a command'
checkvar, com_delim, '!!'
checkvar, com_stack, ''
checkvar, input_line, ''

com_param = ''
error = 0
ipar  = -1 ;indicates no parameter returned

checkvar, nolist, keyword_set(nolist)
checkvar, families, fam_info.fam
;match, fam_info.fam, families, ind
commands = fam_info.commands
commands = commands( uniq(commands, sort(commands)) )
special  =['XDOC','PRINT','HELP','CHKARG','SPAWN']
param_names = prm.nam

if not(nolist) then $
   list_op, prm, fam_info, com_delim, title, families=families


COMMANDS:

if input_line eq '' then begin	
	case 1 of
	com_stack(0) eq '' : begin
		if fcheck(noreadinput,0) then com_line='exit' else begin
		  print,command_prompt
		  com_line = ''
		  read, prompt+'',com_line 		;'' added to
							;make expression
		endelse
	end
	else: begin
		com_line = com_stack(0)
		com_stack = com_stack(1:*)
		wnotblank = where( com_stack ne '', notblank)
		if notblank < 1 then com_stack = [com_stack(wnotblank),'']
	end	
	endcase
endif else begin
	com_line = input_line
	input_line = ''
endelse

;Have a series of commands been passed?
com_stack_top = str2arr( com_line, delim=com_delim)
com_stack    = [com_stack_top, com_stack]
com_line     = com_stack(0)
if n_elements(com_stack) gt 1 then com_stack = com_stack(1:*)

INPUT:
;more,['COM_LINE',com_line]

;return strings w/o blanks in CAPS
;command line assumes strings separated by commas
printx, prompt+com_line, /noprint

com_strings = parse_comline( com_line, nvalues,/nocase) ;mod ras 14-apr-94

com = strupcase(com_strings(0))			;mod ras 14-apr-94
nsearch = strlen(com)

command = '' ;initialize to no command
if nvalues eq 1 then begin; IT'S A COMMAND, MATCH IT UP
	wcom = where( strpos(strupcase(strmid(commands,0,nsearch)),com) ne -1,$
	   ntest)
	if ntest eq 0 then begin

	  wcom = where( strpos(special,com) eq 0 , specialtest)
	  if specialtest eq 0 then wcom = where( strpos(com, '=') ne -1, specialtest)
	  if specialtest eq 1 then begin
		 if com_param(0) ne '' then $
		 com_line = 'idl,'+arr2str([com_line,com_param],',') $
		 else $
		 com_line = 'idl,'+ com_line
		 goto, INPUT
	  endif
	   printx, ' ****** Invalid command ******'
	   goto, error_return
	endif
        if ntest eq 1 or nsearch eq strlen(commands(wcom(0))) then $
	   command = commands(wcom(0)) else begin
              printx,'Ambiguous command.  Supply more letters.'
              goto, error_return
           endelse
	if strupcase(strtrim(command,2)) ne 'EXIT' then printx,string(/print,'Command: ',command)

endif else begin

	wprm = where(strpos(strupcase(strmid(param_names,0,nsearch)),com) ne -1, ntest)
	if ntest eq 0 then begin
		com_param = com_strings(1:*)
		com_line = com_strings(0)
		goto, INPUT
	endif else if ntest gt 1 then begin   ;should match 1 parameter only
			printx, 'Ambiguous parameter.  Supply more letters.'
			goto, error_return 
		endif

	;if 1 parameter matches, then pass back the value within the parameter structure
	ipar =wprm(0)
	svalues = com_strings(1:*)
	if strpos(prm(ipar).nam, 'UT') ne -1  then begin
		ut = utime(arr2str(svalues),error=error)
		if error then goto, error_return
		prm(ipar).val = atime(ut, /yohkoh)
		goto, normal_return
	endif
		

	; Allow for  'param,,value' syntax by testing for blank fields in svalues
	valid = where( svalues ne '', nvalid)
	svalues = svalues(valid)

	; check that parameters are correct type and move to parms
	etest = execute('parms = '+prm(ipar).typ + '(fltarr(nvalid))')
        on_ioerror, error_return
	reads,svalues,parms
        on_ioerror, null
	qnull = where(svalues eq "''", nnull)
        if nnull gt 0 then parms(qnull) = ''

	; check for correct number of parameter values
	if nvalid gt prm(ipar).numval then begin
           printx, string(/print,'Error - maximum number of values allowed = ', $
              prm(ipar).numval)
           goto, error_return
        endif

	; check for parameter values within valid range
        if prm(ipar).min ne -999.d0 then begin
	   q = where (parms lt prm(ipar).min, kq)
           if kq gt 0 then goto, error_range
	endif
        if prm(ipar).max ne -999.d0 then begin
	   q = where (parms gt prm(ipar).max, kq)
           if kq gt 0 then goto, error_range
	endif

	; for strings with options, check that value is one of options
	if prm(ipar).opt(0) ne '' then begin
		q = where(strpos(strupcase(prm(ipar).opt), $
                          strupcase(parms(0))) eq 0, nq)
		if nq eq 0 then begin
                   printx,string(/print,'Error - invalid option.  Options are: ',prm(ipar).opt)
                   goto, error_return
                endif
		prm(ipar).val = prm(ipar).opt(q(0))

	endif else begin

	        prm(ipar).val(valid) = parms

	endelse

endelse
		

goto, NORMAL_RETURN

ERROR_RANGE:
if prm(ipar).min ne -999. and prm(ipar).max ne -999. then $
   printx, string(/print,'Error - Invalid value.  Allowed range is: ',prm(ipar).min, ' - ', $
   prm(ipar).max) $
else if prm(ipar).min ne -999. then printx,string(/print,'Error - Invalid value.  Minimum ',$
   ' value allowed: ',prm(ipar).min) $
else if prm(ipar).max ne -999. then printx,string(/print,'Error - Invalid value.  Maximum ',$
   ' value allowed: ',prm(ipar).max)
goto, error_return

ERROR_RETURN:
print, ' '
printx, string(/print,'  ******   Error on command line: "', com_line, '"')
error=1
goto, getout

NORMAL_RETURN:

GETOUT:

return
end	
