;+  
; PROJECT: SPEX
;
; NAME:   read_batse_4_spex
;
;
; PURPOSE:  general purpose data input proc. for SPEX
;
;
; CATEGORY: SPEX, analysis i/o
;
; CALLS:
;	SPEC_SENSITIVITY, READ_CONT, READ_DISCSP, CONT_EDGES, FCHECK, CHECKVAR,
;	BATSE_DEADTM, F_DIV, DPRATE2SEC, LIMITS, AVE_CTS2, ANYTIM, GT_DP_MODE
;	RESTORE_OVERFLOW CHKARG CLEAR_UTPLOT DET_COSINES EDGE_PRODUCTS FITS_SPECTRA
;	GET_HXRBS_FITS HXT_CAL_FIX PRINTX RD_ROADMAP RD_SXS_PHA RD_WBS_PHA RD_XDA 
;	SHERS_LOAD  SPEX_INTERVALS SXS_ECAL UTPLOT WBS_RESPONSE EXIST ATIME
; 
; CALLING SEQUENCE: 
; read_batse_4_spex, dformat=dformat, files=files, $
; start_time = start_time, end_time=end_time, title=title, command_out=command_out, $
; det_id=det_id, id_style=id_style, p_read_data=p_read_data, _extra=_extra, $
; flux=flux, eflux=eflux, ut=ut, units=units, area=area, ltime=ltime, edges=edges,$
;       id_use=id_use, auto=auto, noplot=noplot, mode=mode, wchan=wchan, delta_light=delta_light
; examples
; 
; read_batse_4_spex, instrument='BATSE', dformat='SHER', det_id=1, id_style='id', id_use=id_use
;         files=['hxrbs::sys$user:[richard.batse.seq_obs]sherb_ibdb.468',flare=665, $
;         'hxrbs::sys$user:[richard.batse.seq_obs]sher_cor_ibdb*468.fits'], $
;         flux=flux, eflux=eflux, ut=ut, units=units, area=area, ltime=ltime, edges=edges
;
;
; INPUTS:
; dformat     - data format:
;                 for BATSE - 'DISCLA','DISCSP','CONT', 'SHERS' (SHER & SHERB), 'HERS' (HER & HERB)
;                  YOHKOH-  HXT, SXS, HXS, GRS
;                  SMM - HXRBS, GRSX, tbd GRS, 
;                        HIREX - GERMANIUM                   
; files       - file or files with the data to be read
; flare       - BATSE archive flare number
; start_time  -  
; end_time    -
;               optional, read file between start_time and end_time, defaults
;               are the start and end of the files, must be readable by ANYITM
;
; det_id      - detector identifier, for BATSE 0-7 
; id_style    - what style of identifier, For BATSE, 'id' - assigned number, default
;                                                    'aspect' - sunward aspect
;                                                    'dsel'   - order in burst readout
; _extra      - keyword inheritance in version >3.1
; /auto       - non-interactive calibration
; /noplot     - no plotting of calibration activities
; p_read_data - name of read procedure for data type not explicitly allowed
; OUTPUTS:
; flux        - fltarr(nchan,nbins) - counts/cm2/sec/keV, overflow and livetime corrected
; eflux       - fltarr(nchan,nbins) - sqrt(counts)/cm2/sec/keV
; ut          - dblarr(2,nbins), seconds referenced to 79/1/1
; units       - e.g. 'cm!u-2!n s!u-1!n keV!u-1!n'
; area        - detector area in cm2
; ltime       - fltarr(1,nbins)  accumulation live time in interval, seconds
;		changed to fltarr(nchan,nbins) - 20-oct-94
; edges       - fltarr(2,nchan)  lo and hi edges in keV
; delta_light - fltarr(nchan), for BATSE SHER, channel width in pulse height
;               see cal_sher and calibrate, and Band 1992 on SLED problem for 
;               discussion.
; id_use      - actual detector id, to reference drm
; title       - title string referencing instrument, data
; mode        - special output for YOHKOH, mode=0 for non-flare, 1 for flare
; wchan       - indices of allowed channels
; command_out - further instructions to SPEX, e.g. "read_drm", may also be used for input
; SIDE EFFECTS:
; none
;
; RESTRICTIONS:
; none
;
; PROCEDURE:
; This procedure provides a 1-stop shop for all BATSE data types readable through SPEX.
; The correct reader is identified for each data type, it is called for the input
; data file.  The correct PHA edge generator is also called.  Corrections are made
; to the data for livetime and overflow and all vectors by SPEX are returned.
;
; MODIFICATION HISTORY: 
; Version 1
;	richard.schwartz@gsfc.nasa.gov, 26-sep-1997, extracted
;	from READ_4_SPEX.PRODISCLA.
; Version 2
;	richard.schwartz@gsfc.nasa.gov, 2-oct-1997, 
;	protects against edges low enough to bomb drm builder
;
;-
pro read_batse_4_spex, dformat=dformat, files=files, $
start_time = start_time, end_time=end_time, flare=flare, $
det_id=det_id, id_style=id_style, p_read_data=p_read_data, $                      
flux=flux, eflux=eflux, ut=ut, units=units, area=area, cosines=cosines, $
ltime=ltime, edges=edges, delta_light=delta_light, wchan=wchan, id_use=id_use, title=title, $
auto=auto, noplot=noplot, mode=mode, error=error, $
command_out=command_out, efficiency = eff, _extra = _extra                        


common read_spex_com, data, ut_data, data_file_read, read_start, read_end
checkvar,data,0
checkvar,data_file_read,''
checkvar,read_start,''
checkvar,read_end,''
checkvar, command_out, ''
if command_out(0) ne '' then com_param=command_out else com_param=''
delta_light= 1.0
checkvar,    units, ' s!u-1!n cm!u-2!n keV!u-1!n'
format = strupcase(dformat(0))

command_out = 'build_drm'     ;BATSE matrices depend on angle of incidence!
data_file = files(0)
if fcheck(flare,0) ne 0 then $
det_cosines, flare=flare, cos_lad=cos_det, cos_spec=cos_spec, /sort, lad_sort=lad_sort,$
spec_sort=spec_sort $
else begin
    checkvar, cosines, [.9,.8,.7,.5,-.5,-.7,-.8,-.9]
    cos_det=cosines
    cos_spec=cosines
    lad_sort=reverse(sort(cosines))
    spec_sort=reverse(sort(cosines))
    endelse
if format eq 'DISCSP' and strlowcase(com_param(0)) eq 'multi' then begin
    wmin_band=where( strpos(com_param,'min') ne -1, nmin_band)
    if nmin_band eq 1 then min_band=float(com_param(wmin_band(0)+1)) 
    woverlay=where( strpos(com_param, 'ov') ne -1, overlap)
    
    discsp_bands, flare, ut, flux, eflux,$
    ltime=ltime, edges=edges, area=area, delta_light=delta_light, $
    filename= files, det_ids=det_ids, title=title, min_band=min_band, overlap=overlap, $
    start_time=start_time, end_time=end_time, error=error, wchan=wchan_use
    data_file_read = files
    nchan = n_elements(wchan_use)
    nbin  = n_elements(ut(0,*))
    edge_products, edges, mean=e
    energy_bands = e([0,0,1,1,2,3,4,6] < wchan_use(nchan-1))
    command_out = ['MULTIDRM',arr2str(['energy_bands',strtrim(energy_bands,2)],',') + $
    '!!graph!! erange, '+strtrim(e(1),2)+','+strtrim(e(wchan_use(nchan-1)),2)]
    endif else begin
    if format eq 'DISCLA' then begin
        det_sorted=lad_sort
        if data_file ne data_file_read or $
        fcheck( start_time,'') ne  read_start or $
        fcheck( end_time,'') ne read_end then begin
            utbase = getutbase()
            ;
            ;Move channel 4 to channel 0, channels 0-3 move to 1-4.
            ;the edges array must be similarly changed when reported out.
            ;
            read_discla, ut_data, data, filename=files(0), /reorder, $
            start=start_time, endt=end_time, live=ltime
            ut_data = ut_data + getutbase()
            setutbase, utbase
            data = {data:data, ltime:ltime}
            endif 
        read_start = ut_data(0)
        read_end = ut_data(n_elements(ut_data)-1)
        idst =strupcase(strmid(id_style,0,1)) 
        if idst eq 'A' then id = lad_sort(id) else $
        id = det_id
        if idst eq 'D' then $
        printx, 'Burst DSEL not applicable to DISCLA data. Using ID'
        printx,string(/print,'Using BATSE LAD DISCLA data from detector ID ',id)
        edges = discla_edges(id) 
        ;
        ;Edges has channel 4 moved to channel 0, channels 0-3 moved to 1-4
        ;by using the id argument.
        ;
        area = 2025.
        title = 'BATSE LAD/DISCLA DET '+STRTRIM(ID,2)+' '+atime(/yoh, ut_data(0),/date)
        wchan_use = indgen(5)
        ut =  ut_data ## (fltarr(2,1)+1.0)
        for i=0,1 do ut(i,*) = ut(i,*) - 0.512 + i*1.024
        
        nchan = n_elements(data.data(*,0,0))
        nbin  = n_elements(data.data(0,0,*))
        ltime = rebin( data.ltime(id,*), nchan, nbin)
        ;
        ; Multiply by ltime because we divide by it later.
        ;
        flux = reform(data.data(*, id, * )) * ltime ;select only 1 detector element
        
        
        endif
    itscont =  where( ['DISCSP', 'CONT'] eq format, nbatse)
    if nbatse eq 1 then begin
        det_sorted=lad_sort
        
        itscont = itscont(0)
        deltat = 2.048
        if data_file ne data_file_read or $
        fcheck( start_time,'') ne  read_start or $
        fcheck( end_time,'') ne read_end then begin
            case itscont of
                1: read_cont, cont=data, seconds=ut_data, file=data_file, $
                start_time=start_time, end_time=end_time
                0: begin
                    read_discsp, discsp=data, seconds=ut_data, file=data_file, $
                    start_time=start_time, end_time=end_time
                    det_sorted=spec_sensitivity(ut_data(0),cos_spec(*,0)) 
                    end
                endcase
            restore_overflow, data, det_sorted
            endif
        data_file_read = data_file
        read_start = fcheck( start_time, '')
        read_end   = fcheck( end_time, '') 
        if not itscont then det_sorted=spec_sensitivity(ut_data(0),cos_spec(*,0)) $
        else det_sorted=lad_sort
        ;if not itscont then det_sorted = spec_sort else det_sorted= lad_sort
        idst =strupcase(strmid(id_style,0,1)) 
        IF itscont then begin
            if idst eq 'A' then id = lad_sort(id) else $
            id = det_id
            if idst eq 'D' then $
            printx, 'Burst DSEL not applicable to CONT data. Using ID'
            printx,string(/print,'Using BATSE LAD CONT data from detector ID ',id)
            set_pendleton, get_pendleton()
            edges = cont_edges(id, time=ut_data(0))
            area = 2025.
            amp_dead = [3.75,3.75,1.05, 3.75+fltarr(5)] * 1e-6
            title = 'BATSE LAD/CONT DET '+STRTRIM(ID,2)+' '+atime(/yoh, ut_data(0),/date)
            wchan = 1 + indgen(15)
            flux = reform(data(*, id, * )) ;select only 1 detector element
            ltime =  float(deltat*batse_deadtm( flux,amp_dead=amp_dead(id) ))
            nchan = n_elements(flux(*,0))
            endif else begin
            if idst eq 'A' then id = spec_sort(id) else           id = det_id
            if idst eq 'D' then $
            printx, 'Burst DSEL not applicable to DISCSP data. Using ID'
            printx,string(/print,'Using BATSE DISCSP data from detector SPEC ID ',id)
            
            edges = (discsp_edges(ut_data(0),det=id))(*,0:1)
            area = 127.
            amp_dead = 1.38e-6 + fltarr(8)
            wchan = [0,1]
            flux = reform(data(*, id, * )) ;select only 1 detector element
            title = 'BATSE DISCSP DET '+STRTRIM(ID,2)+' '+atime(/yoh, ut_data(0),/date)
            channels = [[0,1],[0,1],[0,1],[0,1]]
            for i=0,2 do flux(i,*) = flux(i,*) - flux(i+1,*)
            ltime =  float(deltat*batse_deadtm( flux,amp_dead=amp_dead(id),channel_edg=channels,$
            pha=0.0 ))
            flux  = flux(0:1,*)
            nchan = n_elements(flux(*,0))
            endelse
        nbin = n_elements(flux(0,*))
        ut=transpose( reform([ut_data-deltat/2.,ut_data+deltat/2.],nbin,2))
        endif   ;close CONT & DISCSP & DISCLA selection
    endelse
if format eq 'DISCSP_CONT' then begin
    ;look for use_id in com_param
    ;
    if strlowcase(com_param(0)) eq 'use_id' then use_id = fix( com_param(1:*) )
    merge_batse, flare_info=flare, ut=ut, flux=flux, eflux=eflux,$
    ltime=ltime, edges=edges, area=area, delta_light=delta_light, $
    filenames=filenames, det_ids=det_ids, noflux=noflux,  title=title, $
    start_time=start_time, end_time=end_time, error=error, wchan=wchan_use, use_id=use_id
    data_file_read = filenames(0)
    nchan = n_elements(wchan_use)
    nbin  = n_elements(ut(0,*))
    edge_products, edges, mean=e
    energy_bands = e([0,0,1,1,2,3,4,6])
    command_out = [command_out,arr2str(['energy_bands',strtrim(energy_bands,2)],',') + $
    '!!graph!! erange, '+strtrim(e(1),2)+','+strtrim(e(6),2)]
    endif   ;close DISCSP_CONT 

if format eq 'SHERS' or format eq 'HERS' or $
format eq 'STTE' then begin
    prefx = strmid(format,0,strlen(dformat(0))-1)      ;'HER' OR 'SHER'
    if prefx eq 'STT' then $
    wsher= where(strpos(strupcase(files), 'SHER_') ne -1,nsher) else $
    wsher= where(strpos(strupcase(files), prefx+'_') ne -1,nsher) 
    
    if nsher eq 0 then sher_file='' else sher_file = files(wsher) 
    wsherb= where(strpos(strupcase(files), prefx+'B_') ne -1,nsherb)
    if nsherb eq 0 then sherb_file='' else sherb_file = files(wsherb) 
    if nsherb eq 0 then begin
        wstte= where(strpos(strupcase(files), 'STTE_') ne -1,nstte)
        if nstte eq 1 then  begin
            sherb_file = files(wstte) 
            prefx='STTE'
            endif
        endif
    if n_elements(start_time) ne 0 and n_elements(end_time) ne 0 then $
    timerange = [start_time, end_time]
    
    idst =strupcase(strmid(id_style,0,1)) 
    det_sorted=reverse(sort(cos_spec))
    if idst eq 'A' then id = det_sorted(det_id) else $
    id = det_id
    if idst eq 'D' then $
    printx, 'Burst DSEL not implemented yet for '+prefx+'S, Using ID' 
    if prefx eq 'SHER' then $
    printx,string(/print,'Using BATSE SPEC SHER(B) data from detector ID ',id) $
    else $
    printx,string(/print,'Using BATSE LAD HER(B) data from detector ID ',id )
    
    shers_load, det_id=id, nsher=data, area=area, cos_spec=cos_spec, $
    evec=edges, xvec=xvec, delta_light=delta_light, n511=n511,  $
    sher_file=sher_file(0), sherb_file = sherb_file(0),$
    timerange=timerange, seconds=ut_data, dsel=dsel, ltime=ltime, $
    auto=auto, noplot=noplot
    ;Correct for dead time
    data_file_read = sher_file(0)     ;disrupt test for CONT data re-read
    flux = data
    ut = ut_data
    edges = edges>3.0 ;protect against edge construction yielding values which
		      ;would bomb the drm procedure.	    
    case prefx of
        'STTE': title = 'BATSE SPEC/SHER_STTE DET '+STRTRIM(ID,2)+' '+atime(/yoh, ut(0),/date) 
        'SHER': title = 'BATSE SPEC/SHER(B) DET '+STRTRIM(ID,2)+' '+atime(/yoh, ut(0),/date) 
        else:   title = 'BATSE LAD/HER(B) DET '+STRTRIM(ID,2)+' '+atime(/yoh, ut(0),/date)
        endcase
    nchan = n_elements(flux(*,0))
    nbin = n_elements(flux(0,*))
    endif
id_use = fcheck( id, det_id)
if n_elements(wchan_use) eq 0 then $
case nchan of 
    128: wchan = indgen(117)+7 
    256: wchan = (indgen(256))(where( delta_light gt 0.0) )
    else: if nchan ne 16 then wchan=indgen(nchan)
    endcase else wchan = wchan_use
edge_products, edges, width=ww

;convert to fluxes, counts/s/cm2/keV
if (size(ltime))(0) eq 1 then ltime = rebin( reform(ltime,1,nbin), $
nchan, nbin )
if n_elements(delta_light) le 1 then $
count_2_flux =  ltime * area * rebin(ww(*),nchan,nbin) $
else $
count_2_flux =  ltime * area * rebin(delta_light(*),nchan,nbin) 

eflux = f_div(sqrt(flux), count_2_flux) ;sqrt counts/ factors
flux  = f_div(flux, count_2_flux)       ; counts / factors

end
