;+  
; PROJECT: SPEX
;
; NAME:   read_smm_4_spex
;
;
; PURPOSE:  general purpose data input proc. for SPEX
;
;
; CATEGORY: SPEX, analysis i/o
;
; CALLS:
;	SPEC_SENSITIVITY, READ_CONT, READ_DISCSP, CONT_EDGES, FCHECK, CHECKVAR,
;	F_DIV, DPRATE2SEC, LIMITS, AVE_CTS2, ANYTIM, GT_DP_MODE
;	RESTORE_OVERFLOW CHKARG CLEAR_UTPLOT DET_COSINES EDGE_PRODUCTS FITS_SPECTRA
;	GET_HXRBS_FITS HXT_CAL_FIX PRINTX RD_ROADMAP RD_SXS_PHA RD_WBS_PHA RD_XDA 
;	SHERS_LOAD  SPEX_INTERVALS SXS_ECAL UTPLOT WBS_RESPONSE EXIST ATIME
; 
; CALLING SEQUENCE: 
; read_smm_4_spex, dformat=dformat, files=files, $
; start_time = start_time, end_time=end_time, title=title, command_out=command_out, $
; det_id=det_id, id_style=id_style, p_read_data=p_read_data, _extra=_extra, $
; flux=flux, eflux=eflux, ut=ut, units=units, area=area, ltime=ltime, edges=edges,$
;       id_use=id_use, auto=auto, noplot=noplot, mode=mode, wchan=wchan, delta_light=delta_light
; examples
; 
; read_smm_4_spex,  dformat='HXRBS', id_style='id', id_use=id_use
;         files='$HXRBS_FITS/hxrbs_555.fits', $
;         flux=flux, eflux=eflux, ut=ut, units=units, area=area, ltime=ltime, edges=edges
;
;
; INPUTS:
; dformat     - data format:
;                  SMM-  HXRBS, GRSX, GRS
; files       - file or files with the data to be read
; flare       - hxrbs archive flare number, not implemented
; start_time  -  
; end_time    -
;               optional, read file between start_time and end_time, defaults
;               are the start and end of the files, must be readable by ANYITM
;
; det_id      - detector identifier, for GRS 0-1 
; id_style    - NO MEANING
; _extra      - keyword inheritance in version >3.1
; /auto       - non-interactive calibration
; /noplot     - no plotting of calibration activities
; p_read_data - name of read procedure for data type not explicitly allowed
; OUTPUTS:
; flux        - fltarr(nchan,nbins) - counts/cm2/sec/keV, overflow and livetime corrected
; eflux       - fltarr(nchan,nbins) - sqrt(counts)/cm2/sec/keV
; ut          - dblarr(2,nbins), seconds referenced to 79/1/1
; units       - e.g. 'cm!u-2!n s!u-1!n keV!u-1!n'
; area        - detector area in cm2
; ltime       - fltarr(1,nbins)  accumulation live time in interval, seconds
;		changed to fltarr(nchan,nbins) - 20-oct-94
; edges       - fltarr(2,nchan)  lo and hi edges in keV
; delta_light - fltarr(nchan), for BATSE SHER, channel width in pulse height
;               see cal_sher and calibrate, and Band 1992 on SLED problem for 
;               discussion.
; id_use      - actual detector id, to reference drm
; title       - title string referencing instrument, data
; mode        - special output for YOHKOH, mode=0 for non-flare, 1 for flare
; wchan       - indices of allowed channels
; command_out - further instructions to SPEX, e.g. "read_drm", may also be used for input
; SIDE EFFECTS:
; none
;
; RESTRICTIONS:
; none
;
; PROCEDURE:
; This procedure provides a 1-stop shop for all YOHKOH data types readable through SPEX.
; The correct reader is identified for each data type, it is called for the input
; data file.  The correct PHA edge generator is also called.  Corrections are made
; to the data for livetime and overflow and all vectors by SPEX are returned.
;
; MODIFICATION HISTORY: 
; Version 1
;	richard.schwartz@gsfc.nasa.gov, 26-sep-1997, extracted
;	from read_4_spex
;-
pro read_smm_4_spex, dformat=dformat, files=files, $
start_time = start_time, end_time=end_time, flare=flare, $
det_id=det_id, id_style=id_style, p_read_data=p_read_data, $                      
flux=flux, eflux=eflux, ut=ut, units=units, area=area, cosines=cosines, $
ltime=ltime, edges=edges, delta_light=delta_light, wchan=wchan, id_use=id_use, title=title, $
auto=auto, noplot=noplot, mode=mode, error=error, $
command_out=command_out, efficiency = eff, _extra = _extra                        


common read_spex_com, data, ut_data, data_file_read, read_start, read_end
checkvar,data,0
checkvar,data_file_read,''
checkvar,read_start,''
checkvar,read_end,''
checkvar, command_out, ''
if command_out(0) ne '' then com_param=command_out else com_param=''
delta_light= 1.0
checkvar,    units, ' s!u-1!n cm!u-2!n keV!u-1!n'
format = strupcase(dformat(0))

case format of
    'HXRBS': begin 
        command_out = 'build_drm' 
        get_hxrbs_fits, file=files(0), ut=ut, yarray=flux, eyarray=eflux, $
        /flux, deltat=deltat, edges=edges, livet=ltime
        area=68.6  ;cm^2
        nbin = n_elements(ut)
        edge_products, [ut-deltat/2.,ut(nbin-1)+deltat/2], edges_2=ut
        edge_products, edges, width=ww    
        title = 'SMM HXRBS FLARE '+strtrim(flare,2)+'  '+atime(/yoh, ut(0),/date)
        wchan = indgen(15)
        ; nchan = 15
        end
    'GRSX':  begin
        command_out = 'build_drm' 
        flux = grs_lightcurve( file=files(0), start_time=start_time, $
        end_time=end_time, ut, ltime, edges, label, /xray)
        edges = ([[edges.x1],[edges.x2]]*1.e3)(*,[0,4,1,5,2,3,6,7])
        area = 7.94 ;cm2 for each detector
        nbin =n_elements(ut(0,*))
        flux = (reform(flux, 8, nbin))([0,4,1,5,2,3,6,7],*)
        ltime =  temporary( rebin(reform(ltime,1,2,nbin),4,2,nbin))
        ltime =  temporary( (reform(ltime, 8, nbin))([0,4,1,5,2,3,6,7],*))
        edge_products, edges, width=ww    
        title = 'SMM GRS XRAY1&2 '+atime(/yoh, ut(0),/date)
        wchan = indgen(8)
        nchan = 8
        ;convert to fluxes, counts/s/cm2/keV
        count_2_flux =  ltime * area * rebin(ww(*),nchan,nbin)
        eflux = f_div(sqrt(flux), count_2_flux) ;sqrt counts/ factors
        flux  = f_div(flux, count_2_flux)       ; counts / factors
        end
    'GRS': begin
        command_out = 'build_drm' 
        flux = grs_lightcurve( file=files(0), ut, ltime, edges,/pha_data,$
        start_time=start_time, end_time=end_time  )
        area = 317.55 ;cm2 for each detector
        nbin =n_elements(ut(0,*))
        ltime =  temporary( rebin(reform(ltime,1,nbin),476,nbin))
        nchan = 476
        edges = edges(*,0:nchan-1) * 1e3
        edge_products, edges, width=ww    
        title = 'SMM GRS PHA '+atime(/yoh, ut(0),/date)
        wchan = indgen(200)+10
        ;convert to fluxes, counts/s/cm2/keV
        count_2_flux =  ltime * area * rebin(ww(*),nchan,nbin)
        eflux = f_div(sqrt(flux), count_2_flux) ;sqrt counts/ factors
        flux  = f_div(flux, count_2_flux)       ; counts / factors
        end
    endcase

end
