;+
; PROJECT:
;	SDAC
; 
; NAME: spectral_ratio
;	
; PURPOSE: Find estimates to fitting functions by using
;	ratios of count rates and detector response functions.
;	
; CALLING SEQUENCE:
;	result = SPECTRAL_RATIO( Cflux, DRM=DRM, APAR_IN=APAR_IN, $
;	PHOTON_EDGES=PHOTON_EDGES, [WUSE=WUSE, MODEL=MODEL, $
;	NBIN=NBIN, MRATIO=MRATIO, PAR_OUT=PAR_OUT]
; INPUTS:
;	Cflux - Count rate in output units of DRM.  Frequently both
;	are in cts/cm2/sec/keV.  Dimensioned Channels X Time-Bins.
;	Frequently Time-Bins but may be spatial, too, for which DRM is constant.
;	
; OUTPUTS:	
;	returns the normalization and non-linear parameter for
;	the input count rate and model function for each pair
;	of channels requested in succession, i.e. 0:1, 1:2, 2:3, ...
;
; CATEGORY: SPEX
;
; KEYWORD INPUTS:
; Required:
;	DRM: detector response matrix expressed as counts/chan per photon/cm2
;	PHOTON_EDGES: photon energy edges (keV) used as inputs for cdrm
;	APAR_IN: Input parameters.  2XN.  There may be an arbitrary number
;	of parameters, but there is only 1 which will be interpolated on
;	and the range of interpolation is given by the two values at the ith
;	position.  Its normalization parameter must immediately precede it in
;	the call.  All of the other parameters must have duplicated values and
;	the normalization parameter must be set to 1.0.
;	ECFLUX: one sigma deviations on cflux., dimensioned identically to cflux.
; Optional:
;	MODEL:
;		String of function name model to use. Default is
;		a single power-law.  May be any parametric
;		model, such as F_VTH, in the software tree.
;	NBIN: Number of interpolation bins, 50 default.
; KEYWORD OUTPUTS:
;	MRATIO: Channel ratios for interpolation grid.
;	PAR_OUT: Interpolation parameter vector.	
;	EPAR: Propagated deviations on the return values. Two sets of values 
;	for each return value, for positive and negative excursions of the calculated
;	channel ratio.
; PROCEDURE:
;	creates a grid of ratios for model spectra and interpolates
;
; CALLS:
;	F_DIV
; COMMON BLOCKS:
;	SPECTRAL_RATIO_COM
;
; RESTRICTIONS:
;	none:
;	
; MODIFICATION HISTORY:
;	
;	Version 1	RAS March 8, 1996	
;	Version 2       RAS 25-July-1996, added vector DRM test
;	Version 3, richard.schwartz@gsfc.nasa.gov, 28-jan-1998, cleaned some.
;		Added common block with saved ratios and checks to eliminate recalculating
;		interpolation grid when not needed on successive calls.  Added error propagation.
;
;-
;
function spectral_ratio, cflux, ecflux=ecflux, epar=epar, drm=drm,  apar_in=apar_in, model=model, $
photon_edges=eph, wuse=wuse, nbin=nbin, mratio=mratio, par_out=par_out, $
stopit=stopit


checkvar, model, 'f_bpow'
apar_def =  [[1.,1.],[1.,7.],[max(eph)+[0.,0.]],[1.,1.],[min(eph)+[0.,0.]],[1,1]]
checkvar, apar_in, apar_def
checkvar, wuse, indgen(n_elements(cflux(*,0)))
checkvar, nbin, 50

;
;	Which is the non-linear parameter to vary?
;
ipar=(where(apar_in(0,*) ne apar_in(1,*), npar))(0)

if npar ne 1 then begin
    Print,'Error in Apar_in.  Choose 1 element to vary.'
    print,'Apar_in:', apar_in
    chkarg,'spectral_ratio',proc
    more,proc
    endif	
; It's range is:
par_range = apar_in(*,ipar)	

edge_products, eph, gmean=ephm, wid=weph 

drange = par_range(1)-par_range(0)
par_out = (findgen(nbin)+0.5)*(drange/nbin) + par_range(0)

;
; Check whether MRATIO needs recalculating
;

common spectral_ratio_com, spec_rat_str

test = datatype(spec_rat_str) eq 'STC'
if test then $
test = match_struct(exclude=[0], spec_rat_str, $
{mratio: spec_rat_str.mratio, par_out: par_out, drm:drm(wuse,*), eph:eph,  wuse: wuse, nbin:nbin, apar_in:apar_in})


if not test then begin
    
    apar_use = (apar_in(0,*))(*)#(fltarr(1,nbin)+1.)
    apar_use(ipar,*) = par_out
    apar_use(ipar-1>0,*) = 1.0
    out = fcount_rate( dummy_val, apar_use, eph, drm, wuse=wuse, model=model)
    mratio = f_div( out(0:n_elements(wuse)-2,*),out(1:*,*) )
    
    
    spec_rat_str = {mratio: mratio, par_out: par_out, drm:drm(wuse,*), eph:eph, wuse: wuse, nbin:nbin, apar_in:apar_in}
    
    endif else begin
    mratio = spec_rat_str.mratio
    apar_use = (apar_in(0,*))(*)#(fltarr(1,nbin)+1.)
    apar_use(ipar-1>0,*) = 1.0
    endelse
;
;array of results, norm and param, each ratio, each time
do_errors =  n_elements(ecflux) eq n_elements(cflux) 

out_vals = reform( fltarr( 2, n_elements(cflux(wuse,0))-1, n_elements(cflux(0,*)) ), $
2, n_elements(cflux(wuse,0))-1, n_elements(cflux(0,*)))
if do_errors then epar     = out_vals

for i=1, n_elements(wuse) -1 do begin
    oratio = f_div( cflux(wuse(i-1),*), cflux(wuse(i),*))
    if do_errors then begin
        sratio = oratio * sqrt( f_div(ecflux(wuse(i-1),*)^2,cflux(wuse(i-1),*)^2 + $
        f_div(ecflux(wuse(i),*)^2,cflux(wuse(i),*)^2 )))
        
        endif
    wpos = where( oratio gt 0.0, npos)

    if npos gt 0 then begin
        out_vals(1,i-1,wpos) = interpol( par_out, (mratio(i-1,*))(*), oratio(wpos) )
        
        if do_errors then $
        epar(1,i-1,wpos) = ((interpol( par_out, (mratio(i-1,*))(*), 1.01*oratio(wpos) ) - $
        out_vals(1,i-1,wpos)) * 100.)(*) * sratio(wpos) 
        
        apar_use(ipar,wpos) = out_vals(1,i-1,wpos)
        out_vals(0,i-1,wpos) = f_div( cflux( wuse(i-1), wpos), $
        fcount_rate( dummy_val, apar_use(*,wpos), eph, drm, wuse=wuse(i-1)) )
        endif
    if do_errors then epar(0,i-1,*) = out_vals(0,i-1,*) * f_div(sratio,oratio)

    endfor

if keyword_set(stopit) then stop
if exist(epar) then epar = reform( epar )

return, reform(out_vals)
end
