;+
; Project     : SDAC
;
; Name        : SPEX_DRM_GENX
;
; Purpose     : This procedure loads the DRM from a genx file.
;
; Category    : SPEX, I/O
;
; Explanation : The drm is extracted from a genx file after
;				having been previously prepared.
;				It will be read by the drm reader through this call:
;
;			call_procedure, p_drm_reader, file=drm_to_read, sfile=scatter_to_read, $
;			edges_out=edges_out_drm, edges_in=e_in, area=area, drm=drm
;
; Syntax      :
;				Spex_drm_genx, $
;					FILE=file, $
;					DRM=drm, $
;					EDGES_IN=edges_in, $
;					EDGES_OUT=edges_out, $
;					AREA=area,  $
;					SFILE=sfile
;
; Examples    :
;
; Inputs      :
;
; Opt. Inputs :
;
; Opt. Outputs:
;
; Keywords    :
; files       - file or files with the data to be read
; start_time  -
; end_time    -
;               optional, read file between start_time and end_time, defaults
;               are the start and end of the files, must be readable by ANYITM
;
; Keyword Outputs:
;
; flux        - fltarr(nchan,nbins) - counts/cm2/sec/keV, overflow and livetime corrected
; eflux       - fltarr(nchan,nbins) - sqrt(counts)/cm2/sec/keV
; ut          - dblarr(2,nbins), seconds referenced to 79/1/1
; units       - e.g. 'cm!u-2!n s!u-1!n keV!u-1!n'
; area        - detector area in cm2
; ltime       - fltarr(1,nbins)  accumulation live time in interval, seconds
;               changed to fltarr(nchan,nbins) - 20-oct-94
; edges       - fltarr(2,nchan)  lo and hi edges in keV
; delta_light - fltarr(nchan), e.g. for BATSE SHER, channel width in pulse height
;               see cal_sher and calibrate, and Band 1992 on SLED problem for
;               discussion.  This allows channel widths for normalization (counts/keV)
;		to be different from channel edges.
; id_use      - actual detector id, to reference drm
; title       - title string referencing instrument, data
; mode        - special output for YOHKOH, mode=0 for non-flare, 1 for flare
; wchan       - indices of allowed channels
; AUTO        - further instructions to SPEX, e.g. "read_drm", may also be used for input
;
; Calls	      :
;		LOC_FILE, RESTGEN, COPY_STRUCT, ANYTIM, FIND_IX, GET_TAG_VALUE
; Common      :
;
; Restrictions:
;
; Side effects: P_DRM_READER, a string with a procedure name for building the DRM, may
;	be set if the tag P_DRM exists in the structure of the first arg. of the genx
;	file in files(0).
;
; History     : Version 1, RAS, 11-Jul-1997
;
; Contact     : Richard.Schwartz@gsfc.nasa.gov
;-
pro spex_drm_genx, file=file, drm=drm, area=area, $
        edges_out=edges, edges_in=ph_edges, sfile=sfile, error=error
error = 1

filename = loc_file(file(0), path=[curdir(),'SPEX_DATA'], count=count)
if count eq 0 then return

restgen, file=filename, spex_data, drm


area = get_tag_value(spex_data, /area, /quiet)
edges= get_tag_value(spex_data, /edges, /quiet)
edge_products, get_tag_value( spex_data,/ph_edges, /quiet ), edges_2=ph_edges

error=0

end

